#include "dEvian.h"

static E_Config_DD *_devian_config_main_edd_init(E_Config_DD *o);
static E_Config_DD *_devian_config_devians_edd_init();

/* Public functions */

Config_dEvian_Main *devian_config_init(void)
{
  Config_dEvian_Main *conf;

  dEvianM->conf_edd_devians = _devian_config_devians_edd_init();
  if (!dEvianM->conf_edd_devians)
    return NULL;
  dEvianM->conf_edd = _devian_config_main_edd_init(dEvianM->conf_edd_devians);
  if (!dEvianM->conf_edd)
    return NULL;

  conf = devian_config_load(dEvianM->conf_edd);
  if (!conf)
    return NULL;

  return conf;
}

Config_dEvian_Main *devian_config_load(E_Config_DD *cfg_main_edd)
{
  Config_dEvian_Main *cfg_main;
  
  cfg_main = e_config_domain_load("module.devian", cfg_main_edd);

  /* If loaded, check for good version */
  if (cfg_main)
    {
      if (cfg_main->conf_version < CONFIG_VERSION)
	{  
	  char buf[4096];
	  snprintf(buf, sizeof(buf),
		   _(
		     "<title>Configuration Upgraded</title><br>"
		     "Configuration v. needed : %d<br>"
		     "Your config file v. : %d<br><br>"
		     "Your old configuration of dEvian module<br>"
		     "has been upgraded<br>"
		     "Your settings were removed<br>"
		     "Sorry for the inconvenience"),
		   CONFIG_VERSION,
		   cfg_main->conf_version);
	  e_module_dialog_show(_("dEvian Module"), buf);
	  
	  cfg_main = NULL;
	}
      else
	{
	  if (cfg_main->conf_version > CONFIG_VERSION)
	    {
	      char buf[4096];
	      snprintf(buf, sizeof(buf),
		       _(
			 "<title>Configuration Downgraded</title><br>"
			 "Configuration v. needed : %d<br>"
			 "Your config file v. : %d<br><br>"
			 "Your old configuration of dEvian module<br>"
			 "has been downgraded<br>"
			 "Your settings were removed<br>"
			 "Sorry for the inconvenience"),
		       CONFIG_VERSION,
		       cfg_main->conf_version);
	      e_module_dialog_show(_("dEvian Module"), buf);
	      cfg_main = NULL;
	    }
	}
    }

  if (!cfg_main)
    cfg_main = devian_config_main_new();

  if (cfg_main->nb_devian != evas_list_count(cfg_main->devians_conf))
    {
      fprintf(stderr, "dEvian: nb_devian (%d) != count(devians_conf) (%d) !!!\n",
	      cfg_main->nb_devian, evas_list_count(cfg_main->devians_conf));
      cfg_main->nb_devian = evas_list_count(cfg_main->devians_conf);
    }

  E_CONFIG_LIMIT(cfg_main->high_quality, 0, 1);
  //... ADD all config limits

  DMAIN(("dEvian: config initialised"));

  return cfg_main;
}

void devian_config_save(void)
{
  Config_dEvian_Main *cfg_main;
  Evas_List *l;
  dEvian *devian;

  cfg_main = dEvianM->conf;
  if (!cfg_main)
    return;

  /* Save number of devians */
  cfg_main->nb_devian = evas_list_count(dEvianM->devians);

  /* Save config of each devian in main config */
  cfg_main->devians_conf = NULL;
  for (l = dEvianM->devians; l; l=evas_list_next(l))
    {
      devian = evas_list_data(l);
      devian_config_devian_save(devian);
      cfg_main->devians_conf = evas_list_append(cfg_main->devians_conf,
						devian->conf);
      DMAIN(("Config saved"));
    }
}

void devian_config_devian_save(dEvian *devian)
{
  Container_Box *box;

  if (!devian->conf)
    return;

  if (devian->conf->container_type == CONTAINER_BOX)
    {
      box = devian->container;
      devian->conf->box_x = box->x;
      devian->conf->box_y = box->y;
    }
}


void devian_config_free(void)
{
  Evas_List *l;

  Config_dEvian_Main *cfg_main;

  cfg_main = dEvianM->conf;

  evas_stringshare_del(cfg_main->theme);
  evas_stringshare_del(cfg_main->sources_picture_data_import_dir);

  for (l = dEvianM->devians; l; l=evas_list_next(l))
    {
      devian_config_devian_free(evas_list_data(l));
    }

  E_FREE(dEvianM->conf);

  E_CONFIG_DD_FREE(dEvianM->conf_edd);
  E_CONFIG_DD_FREE(dEvianM->conf_edd_devians);
}

void devian_config_devian_free(dEvian *devian)
{
  if (!devian->conf)
    return;

  E_FREE(devian->conf);
  devian->conf = NULL;
}

Config_dEvian_Main *devian_config_main_new(void)
{
  Config_dEvian_Main *cfg_main;
  char buf[DEVIAN_MAX_PATH];

  /* Create new config */
  DMAIN(("dEvian: New config creation ..."));
  cfg_main = E_NEW(Config_dEvian_Main, 1);
  cfg_main->conf_version = CONFIG_VERSION;
  cfg_main->high_quality = 0;
  cfg_main->nb_devian = 0;
  strcpy(buf, "simple");
  cfg_main->theme = (char *)evas_stringshare_add(buf);
  
  cfg_main->source_default = SOURCE_PICTURE;
  snprintf(buf, DEVIAN_MAX_PATH, "%s/Tofs", e_user_homedir_get());
  cfg_main->sources_picture_data_import_dir = (char *)evas_stringshare_add(buf);
  cfg_main->sources_picture_data_import_thumbs_warn = 1;
  cfg_main->sources_picture_data_import_recursive = 0;
  cfg_main->sources_picture_default_location = DATA_PICTURE_LOCAL;
  cfg_main->sources_picture_set_bg_purge = SOURCE_PICTURE_SET_BG_PURGE_DEFAULT;
  
  cfg_main->container_default = CONTAINER_BOX;
  cfg_main->boxs_always_number = CONTAINER_BOX_ALWAYS_NUMBER_DEFAULT;
  cfg_main->boxs_always_infos = CONTAINER_BOX_ALWAYS_INFOS_DEFAULT;
  cfg_main->boxs_stacking = CONTAINER_BOX_STACKING_DEFAULT;
  cfg_main->boxs_default_max_size = CONTAINER_BOX_SIZE_DEFAULT;
  cfg_main->boxs_default_anim = CONTAINER_BOX_ANIM_DEFAULT;
  cfg_main->boxs_anim_ghost_timer = CONTAINER_BOX_ANIM_GHOST_TIMER_DEFAULT;
  cfg_main->boxs_nice_resize = CONTAINER_BOX_NICE_RESIZE_DEFAULT;
  cfg_main->boxs_nice_trans = CONTAINER_BOX_NICE_TRANS_DEFAULT;

  cfg_main->data_picture_thumb_default_size = DATA_PICTURE_THUMB_SIZE_DEFAULT;
  cfg_main->data_picture_cache_size = DATA_PICTURE_CACHE_SIZE_DEFAULT;
  
  cfg_main->devians_conf = NULL;

  return cfg_main;
}

/*
  Creates a new config for a devian
  Args:
    - Conf: if !NULL, the returned config will be a copy of this one
  Return:
    The new config
*/
Config_dEvian *devian_config_devian_new(Config_dEvian *conf)
{
  Config_dEvian *cfg;

  cfg = E_NEW(Config_dEvian, 1);

  if (conf)
    {
      cfg->source_type = conf->source_type;
      cfg->picture_timer_active = conf->picture_timer_active;
      cfg->picture_timer_s = conf->picture_timer_s;

      cfg->container_type = conf->container_type;
      cfg->box_speed = conf->box_speed;
      cfg->box_max_size = conf->box_max_size;
      cfg->box_x = conf->box_x;
      cfg->box_y = conf->box_y;
      cfg->box_auto_resize = conf->box_auto_resize;
      cfg->box_anim = conf->box_anim;
    }
  else
    {
      cfg->source_type = dEvianM->conf->source_default;
      cfg->picture_timer_active = 1;
      cfg->picture_timer_s = SOURCE_PICTURE_TIMER_DEFAULT;
      
      cfg->container_type = dEvianM->conf->container_default;
      cfg->box_speed = CONTAINER_BOX_SPEED_DEFAULT;
      cfg->box_max_size = dEvianM->conf->boxs_default_max_size;
      devian_container_box_random_pos_get(&cfg->box_x, &cfg->box_y,
					  cfg->box_max_size);
      cfg->box_auto_resize = CONTAINER_BOX_AUTO_RESIZE_DEFAULT;
      cfg->box_anim = dEvianM->conf->boxs_default_anim;
    }

  DMAIN(("New config created"));

  return cfg;
}

/* Private functions */

static E_Config_DD *_devian_config_main_edd_init(E_Config_DD *devians)
{
  E_Config_DD *edd_main;

  edd_main = E_CONFIG_DD_NEW("dEvian_Config", Config_dEvian_Main);

#undef T
#undef D
#define T Config_dEvian_Main
#define D edd_main
  
  E_CONFIG_VAL(D, T, conf_version, INT);
  E_CONFIG_VAL(D, T, high_quality, INT);
  E_CONFIG_VAL(D, T, nb_devian, INT);
  E_CONFIG_VAL(D, T, theme, STR);


  E_CONFIG_VAL(D, T, source_default, INT);

  E_CONFIG_VAL(D, T, sources_picture_data_import_dir, STR);
  E_CONFIG_VAL(D, T, sources_picture_data_import_thumbs_warn, INT);
  E_CONFIG_VAL(D, T, sources_picture_data_import_recursive, INT);
  E_CONFIG_VAL(D, T, sources_picture_default_location, INT);
  E_CONFIG_VAL(D, T, sources_picture_set_bg_purge, INT);


  E_CONFIG_VAL(D, T, container_default, INT);

  E_CONFIG_VAL(D, T, boxs_always_number, INT);
  E_CONFIG_VAL(D, T, boxs_always_infos, INT);
  E_CONFIG_VAL(D, T, boxs_stacking, INT);
  E_CONFIG_VAL(D, T, boxs_default_max_size, INT);
  E_CONFIG_VAL(D, T, boxs_default_anim, INT);
  E_CONFIG_VAL(D, T, boxs_anim_ghost_timer, INT);
  E_CONFIG_VAL(D, T, boxs_nice_resize, INT);
  E_CONFIG_VAL(D, T, boxs_nice_trans, INT);

  E_CONFIG_VAL(D, T, data_picture_thumb_default_size, INT);
  E_CONFIG_VAL(D, T, data_picture_cache_size, INT);

  E_CONFIG_LIST(D, T, devians_conf, devians);
   
  return edd_main;
}

static E_Config_DD *_devian_config_devians_edd_init(void)
{
  E_Config_DD *devians;

  devians = E_CONFIG_DD_NEW("dEvian_Config_dEvians", Config_dEvian);

#undef T
#undef D
#define T Config_dEvian
#define D devians

  E_CONFIG_VAL(D, T, source_type, INT);
  E_CONFIG_VAL(D, T, picture_timer_active, INT);
  E_CONFIG_VAL(D, T, picture_timer_s, INT);

  E_CONFIG_VAL(D, T, container_type, INT);

  E_CONFIG_VAL(D, T, box_speed, INT);
  E_CONFIG_VAL(D, T, box_x, INT);
  E_CONFIG_VAL(D, T, box_y, INT);
  E_CONFIG_VAL(D, T, box_max_size, INT);
  E_CONFIG_VAL(D, T, box_auto_resize, INT);
  E_CONFIG_VAL(D, T, box_anim, INT);

  return devians;
}
