#include "dEvian.h"

typedef struct _MainCFData MainCFData;
typedef struct _dEvianCFData dEvianCFData;

static void *_main_create_data(E_Config_Dialog *cfd);
static void _main_free_data(E_Config_Dialog *cfd, MainCFData *cfdata);
static int _main_basic_apply_data(E_Config_Dialog *cfd, MainCFData *cfdata);
static int _main_advanced_apply_data(E_Config_Dialog *cfd, MainCFData *cfdata);
static Evas_Object *_main_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, MainCFData *cfdata);
static Evas_Object *_main_advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, MainCFData *cfdata);

static void *_devian_create_data(E_Config_Dialog *cfd);
static void _devian_free_data(E_Config_Dialog *cfd, dEvianCFData *cfdata);
static int _devian_basic_apply_data(E_Config_Dialog *cfd, dEvianCFData *cfdata);
static int _devian_advanced_apply_data(E_Config_Dialog *cfd, dEvianCFData *cfdata);
static Evas_Object *_devian_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, dEvianCFData *cfdata);
static Evas_Object *_devian_advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, dEvianCFData *cfdata);

/* Actual config data we will be playing with whil the dialog is active */
struct _MainCFData
{
  /*- BASIC -*/
  int nb_devian;
  int boxs_always_number;
  int boxs_always_infos;
  const char *sources_picture_data_import_dir;
  int sources_picture_data_import_recursive;
  /*- ADVANCED -*/
  int boxs_nice_resize;
  int boxs_nice_trans;
  int boxs_default_max_size;
  int boxs_default_anim;
  int boxs_anim_ghost_timer;
  int data_picture_thumb_default_size;
  int sources_picture_set_bg_purge;
};
struct _dEvianCFData
{
  Config_dEvian *d_conf;
  Container_Box *c_box;
  Source_Picture *s_picture;
  /*- BASIC -*/
  int container_box_size;
  int container_box_animation;
  /*- ADVANCED -*/
  int container_box_speed;
  int container_box_auto_resize;
  int source_picture_timer;
  int source_picture_timer_yn;
};

/* PUBLIC FUNCTIONS */

E_Config_Dialog *
devian_config_dialog_main(E_Container *con)
{
  E_Config_Dialog *cfd;
  E_Config_Dialog_View v;

  /* methods */
  v.create_cfdata           = _main_create_data;
  v.free_cfdata             = _main_free_data;
  v.basic.apply_cfdata      = _main_basic_apply_data;
  v.basic.create_widgets    = _main_basic_create_widgets;
  v.advanced.apply_cfdata   = _main_advanced_apply_data;
  v.advanced.create_widgets = _main_advanced_create_widgets;
  /* create config diaolg for NULL object/data */
  cfd = e_config_dialog_new(con, _("dEvian Module Configuration"), NULL, 0, &v, NULL);
  return cfd;
}

void devian_config_dialog_main_shutdown(void)
{
  if (dEvianM->dialog_conf)
    while ( e_object_unref(E_OBJECT(dEvianM->dialog_conf)) > 0 );
}

E_Config_Dialog *
devian_config_dialog_devian(E_Container *con, dEvian *devian)
{
  E_Config_Dialog *cfd;
  E_Config_Dialog_View v;

  /* methods */
  v.create_cfdata           = _devian_create_data;
  v.free_cfdata             = _devian_free_data;
  v.basic.apply_cfdata      = _devian_basic_apply_data;
  v.basic.create_widgets    = _devian_basic_create_widgets;
  v.advanced.apply_cfdata   = _devian_advanced_apply_data;
  v.advanced.create_widgets = _devian_advanced_create_widgets;
  /* create config diaolg for NULL object/data */
  cfd = e_config_dialog_new(con, _("dEvian Module Configuration"), NULL, 0, &v, devian);
  return cfd;
}

void devian_config_dialog_devian_shutdown(dEvian *devian)
{
  if(devian->dialog_conf)
    while ( e_object_unref(E_OBJECT(devian->dialog_conf)) > 0 );
}


/* PRIVATE FUNCTIONS */

/* Main dEvian part */

/**--CREATE--**/
static void
_main_fill_data(MainCFData *cfdata)
{
  Config_dEvian_Main *conf;
  conf = dEvianM->conf;

  cfdata->nb_devian = evas_list_count(dEvianM->devians);
  if (!cfdata->sources_picture_data_import_dir)
    cfdata->sources_picture_data_import_dir =
      evas_stringshare_add(conf->sources_picture_data_import_dir);
  cfdata->sources_picture_data_import_recursive =
    conf->sources_picture_data_import_recursive;

  cfdata->boxs_always_number = conf->boxs_always_number;
  cfdata->boxs_always_infos = conf->boxs_always_infos;
  cfdata->boxs_nice_resize = conf->boxs_nice_resize;
  cfdata->boxs_nice_trans = conf->boxs_nice_trans;
  cfdata->boxs_default_max_size = conf->boxs_default_max_size;
  cfdata->boxs_default_anim = conf->boxs_default_anim;
  cfdata->boxs_anim_ghost_timer = conf->boxs_anim_ghost_timer;
  cfdata->data_picture_thumb_default_size =
    conf->data_picture_thumb_default_size;
  cfdata->sources_picture_set_bg_purge =
    conf->sources_picture_set_bg_purge;
}

static void *
_main_create_data(E_Config_Dialog *cfd)
{
  /* Create cfdata - cfdata is a temporary block of config data that this
   * dialog will be dealing with while configuring. it will be applied to
   * the running systems/config in the apply methods
   */
  MainCFData *cfdata;
   
  cfdata = E_NEW(MainCFData, 1);
  cfdata->sources_picture_data_import_dir = NULL;
  
  _main_fill_data(cfdata);
  return cfdata;
}

static void
_main_free_data(E_Config_Dialog *cfd, MainCFData *cfdata)
{
  /* Free the cfdata */
  evas_stringshare_del(cfdata->sources_picture_data_import_dir);
  free(cfdata);
}

/**--APPLY--**/
static int
_main_basic_apply_data(E_Config_Dialog *cfd, MainCFData *cfdata)
{
  /* Actually take our cfdata settings and apply them in real life */
  int i, j;
  Evas_List *l;
  dEvian *devian;
  
  evas_stringshare_del(dEvianM->conf->sources_picture_data_import_dir);
  dEvianM->conf->sources_picture_data_import_dir = 
    (char *)evas_stringshare_add(cfdata->sources_picture_data_import_dir);
dEvianM->conf->sources_picture_data_import_recursive =
  cfdata->sources_picture_data_import_recursive;

  if ((dEvianM->conf->boxs_always_number != cfdata->boxs_always_number) ||
      (dEvianM->conf->boxs_always_infos != cfdata->boxs_always_infos))
    {
      dEvianM->conf->boxs_always_number = cfdata->boxs_always_number;
      dEvianM->conf->boxs_always_infos = cfdata->boxs_always_infos;
      for(l = dEvianM->devians; l; l=evas_list_next(l))
	{
	  devian = evas_list_data(l);
	  if (devian->conf->container_type == CONTAINER_BOX)
	    {
	      devian_container_box_infos_display_set(devian->container);
	    }
	}
    }
  
  /* dEvian creation (last) */
  i = evas_list_count(dEvianM->devians) - cfdata->nb_devian;
  if (i<0)
    for(j=i; j<0; j++)
      devian_devian_add(NULL);
  if (i>0)
    {
      l = evas_list_last(dEvianM->devians);
      for (j=i; j>0; j--)
	{
	  devian_devian_del( evas_list_nth(dEvianM->devians, j-1), 0 );
	}
    }

  return 1; /* Apply was OK */
}

static int
_main_advanced_apply_data(E_Config_Dialog *cfd, MainCFData *cfdata)
{
  /* Actually take our cfdata settings and apply them in real life */
  int i, j;
  Evas_List *l;
  dEvian *devian;

  evas_stringshare_del(dEvianM->conf->sources_picture_data_import_dir);
  dEvianM->conf->sources_picture_data_import_dir = 
    (char *)evas_stringshare_add(cfdata->sources_picture_data_import_dir);
  dEvianM->conf->sources_picture_data_import_recursive =
    cfdata->sources_picture_data_import_recursive;

  if ((dEvianM->conf->boxs_always_number != cfdata->boxs_always_number) ||
      (dEvianM->conf->boxs_always_infos != cfdata->boxs_always_infos))
    {
      dEvianM->conf->boxs_always_number = cfdata->boxs_always_number;
      dEvianM->conf->boxs_always_infos = cfdata->boxs_always_infos;
      for(l = dEvianM->devians; l; l=evas_list_next(l))
	{
	  devian = evas_list_data(l);
	  if (devian->conf->container_type == CONTAINER_BOX)
	    {
	      devian_container_box_infos_display_set(devian->container);
	    }
	}
    }

  /* Advanced only */
  dEvianM->conf->boxs_nice_resize = cfdata->boxs_nice_resize;
  dEvianM->conf->boxs_nice_trans = cfdata->boxs_nice_trans;
  dEvianM->conf->boxs_default_max_size = cfdata->boxs_default_max_size;
  dEvianM->conf->boxs_default_anim = cfdata->boxs_default_anim;
  if (dEvianM->conf->boxs_anim_ghost_timer != cfdata->boxs_anim_ghost_timer)
    {
      dEvianM->conf->boxs_anim_ghost_timer = cfdata->boxs_anim_ghost_timer;
      for(l = dEvianM->devians; l; l=evas_list_next(l))
	{
	  devian = evas_list_data(l);
	  if (devian->conf->container_type == CONTAINER_BOX)
	    {
	      devian_container_box_animation_start(devian->container,
						   CONTAINER_BOX_ANIM_GHOST);
	    }
	}
    }
  dEvianM->conf->data_picture_thumb_default_size = cfdata->data_picture_thumb_default_size;
  dEvianM->conf->sources_picture_set_bg_purge = cfdata->sources_picture_set_bg_purge;

  /* dEvians creation (last) */
  i = evas_list_count(dEvianM->devians) - cfdata->nb_devian;
  if (i<0)
    for(j=i; j<0; j++)
      devian_devian_add(NULL);
  if (i>0)
    {
      l = evas_list_last(dEvianM->devians);
      for (j=i; j>0; j--)
	{
	  devian_devian_del( evas_list_nth(dEvianM->devians, j-1), 0 );
	}
    }

  return 1; /* Apply was OK */
}

/**--GUI--**/
static Evas_Object *
_main_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, MainCFData *cfdata)
{
  /* generate the core widget layout for a basic dialog */
  Evas_Object *o, *of, *ob;
   
  _main_fill_data(cfdata);
   
  o = e_widget_table_add(evas, 0);
   
  of = e_widget_frametable_add(evas, _("dEvians"), 0);
  ob = e_widget_label_add(evas, _("Number"));
  e_widget_frametable_object_append(of, ob,
				    0, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_slider_add(evas, 1, 0, _("%1.0f"), 0.0, 15.0, 1.0, 0,
			   NULL, &(cfdata->nb_devian), 150);
  e_widget_frametable_object_append(of, ob,
				    0, 1, 1, 1,
				    1, 0, 1, 0);
  e_widget_table_object_append(o, of,
			       0, 0, 1, 1,
			       1, 1, 1, 1);

  of = e_widget_frametable_add(evas, _("Boxs"), 0);
  ob = e_widget_check_add(evas, _("Always number"), &(cfdata->boxs_always_number));
  e_widget_frametable_object_append(of, ob,
				    0, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_check_add(evas, _("Always infos"), &(cfdata->boxs_always_infos));
  e_widget_frametable_object_append(of, ob,
				    1, 0, 1, 1,
				    1, 1, 1, 1);
  e_widget_table_object_append(o, of,
			       0, 1, 1, 1,
			       1, 1, 1, 1);

  of = e_widget_frametable_add(evas, _("Pictures"), 0);
  ob = e_widget_label_add(evas, _("Import dir"));
  e_widget_frametable_object_append(of, ob,
				    0, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_entry_add(evas, &(cfdata->sources_picture_data_import_dir));
  e_widget_min_size_set(ob, 100, 1);
  e_widget_frametable_object_append(of, ob,
				    1, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_check_add(evas, _("Load sub-directories"), &(cfdata->sources_picture_data_import_recursive));
  e_widget_frametable_object_append(of, ob,
				    0, 1, 2, 1,
				    1, 1, 1, 1);
  ob = e_widget_button_add(evas, "Regenerate pictures list", NULL, devian_data_picture_list_local_regen, NULL, NULL);
  e_widget_frametable_object_append(of, ob,
				    0, 2, 2, 1,
				    1, 1, 1, 1);
  e_widget_table_object_append(o, of,
			       1, 1, 1, 1,
			       1, 1, 1, 1);

  return o;
}

static Evas_Object *
_main_advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, MainCFData *cfdata)
{
  /* generate the core widget layout for an advanced dialog */
  Evas_Object *o, *of, *ob;
  E_Radio_Group *rg;
   
  _main_fill_data(cfdata);
   

  o = e_widget_table_add(evas, 0);
   
  of = e_widget_frametable_add(evas, _("dEvians"), 0);
  ob = e_widget_label_add(evas, _("Number"));
  e_widget_frametable_object_append(of, ob,
				    0, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_slider_add(evas, 1, 0, _("%1.0f"), 0.0, 15.0, 1.0, 0,
			   NULL, &(cfdata->nb_devian), 150);
  e_widget_frametable_object_append(of, ob,
				    0, 1, 1, 1,
				    1, 0, 1, 0);
  e_widget_table_object_append(o, of,
			       0, 0, 1, 1,
			       1, 1, 1, 1);

  of = e_widget_frametable_add(evas, _("Boxs"), 0);
  ob = e_widget_check_add(evas, _("Always number"), &(cfdata->boxs_always_number));
  e_widget_frametable_object_append(of, ob,
				    0, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_check_add(evas, _("Always infos"), &(cfdata->boxs_always_infos));
  e_widget_frametable_object_append(of, ob,
				    1, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_check_add(evas, _("Nice resizes"), &(cfdata->boxs_nice_resize));
  e_widget_frametable_object_append(of, ob,
				    0, 1, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_check_add(evas, _("Nice transitions"), &(cfdata->boxs_nice_trans));
  e_widget_frametable_object_append(of, ob,
				    1, 1, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_label_add(evas, _("Creation size"));
  e_widget_frametable_object_append(of, ob,
				    0, 2, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_slider_add(evas, 1, 0, _("%1.0f pixels"), 100.0, 500.0, 1.0, 0,
			   NULL, &(cfdata->boxs_default_max_size), 100);
  e_widget_frametable_object_append(of, ob,
				    1, 2, 1, 1,
				    1, 0, 1, 0);
  ob = e_widget_label_add(evas, _("Creation anim"));
  e_widget_frametable_object_append(of, ob,
				    0, 3, 1, 1,
				    1, 1, 1, 1);
  rg = e_widget_radio_group_new(&(cfdata->boxs_default_anim));
  ob = e_widget_radio_add(evas, _("No"), CONTAINER_BOX_ANIM_NO, rg);
  e_widget_frametable_object_append(of, ob,
				    1, 3, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_radio_add(evas, _("Line"), CONTAINER_BOX_ANIM_LINE, rg);
  e_widget_frametable_object_append(of, ob,
				    1, 4, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_radio_add(evas, _("Gouloum"), CONTAINER_BOX_ANIM_GOULOUM, rg);
  e_widget_frametable_object_append(of, ob,
				    1, 5, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_radio_add(evas, _("Ghost"), CONTAINER_BOX_ANIM_GHOST, rg);
  e_widget_frametable_object_append(of, ob,
				    1, 6, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_label_add(evas, _("Ghost anim timer"));
  e_widget_frametable_object_append(of, ob,
				    0, 7, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_slider_add(evas, 1, 0, _("%1.0fs"), 7.0, 120.0, 1.0, 0,
			   NULL, &(cfdata->boxs_anim_ghost_timer), 150);
  e_widget_frametable_object_append(of, ob,
				    1, 7, 1, 1,
				    1, 0, 1, 0);
  e_widget_table_object_append(o, of,
			       0, 1, 1, 1,
			       1, 1, 1, 1);

  of = e_widget_frametable_add(evas, _("Pictures"), 0);
  ob = e_widget_label_add(evas, _("Import dir"));
  e_widget_frametable_object_append(of, ob,
				    0, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_entry_add(evas, &(cfdata->sources_picture_data_import_dir));
  e_widget_min_size_set(ob, 100, 1);
  e_widget_frametable_object_append(of, ob,
				    1, 0, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_check_add(evas, _("Load sub-directories"), &(cfdata->sources_picture_data_import_recursive));
  e_widget_frametable_object_append(of, ob,
				    0, 1, 2, 1,
				    1, 1, 1, 1);
  ob = e_widget_label_add(evas, _("Quality"));
  e_widget_frametable_object_append(of, ob,
				    0, 2, 1, 1,
				    1, 1, 1, 1);
  ob = e_widget_slider_add(evas, 1, 0, _("%1.0f"), 100.0, 500.0, 1.0, 0,
			   NULL, &(cfdata->data_picture_thumb_default_size), 80);
  e_widget_frametable_object_append(of, ob,
				    1, 2, 1, 1,
				    1, 0, 1, 0);
  ob = e_widget_check_add(evas, _("Remove created .e/e/backgrounds/"), &(cfdata->sources_picture_set_bg_purge));
  e_widget_frametable_object_append(of, ob,
				    0, 3, 2, 1,
				    1, 1, 1, 1);
  ob = e_widget_button_add(evas, "Regenerate pictures list", NULL, devian_data_picture_list_local_regen, NULL, NULL);
  e_widget_frametable_object_append(of, ob,
				    0, 4, 2, 1,
				    1, 1, 1, 1);
  e_widget_table_object_append(o, of,
			       1, 1, 1, 1,
			       1, 1, 1, 1);

  return o;
}


/* dEvian part */

/**--CREATE--**/
static void
_devian_fill_data(dEvianCFData *cfdata)
{
  Config_dEvian *d_conf;
  d_conf = cfdata->d_conf;
  
  if (cfdata->c_box)
    {
      cfdata->container_box_size = d_conf->box_max_size;
      cfdata->container_box_auto_resize = d_conf->box_auto_resize;
      cfdata->container_box_animation = d_conf->box_anim;
      cfdata->container_box_speed = d_conf->box_speed;
    }

  if (cfdata->s_picture)
    {
      cfdata->source_picture_timer_yn = d_conf->picture_timer_active;
      cfdata->source_picture_timer = d_conf->picture_timer_s;
    }
}

static void *
_devian_create_data(E_Config_Dialog *cfd)
{
  /* Create cfdata - cfdata is a temporary block of config data that this
   * dialog will be dealing with while configuring. it will be applied to
   * the running systems/config in the apply methods
   */
  dEvianCFData *cfdata;
  dEvian *devian;
  
  devian = cfd->data;
  cfdata = E_NEW(dEvianCFData, 1);

  cfdata->d_conf = devian->conf;
  cfdata->c_box = NULL;
  cfdata->s_picture = NULL;
  
  if (devian->conf->container_type == CONTAINER_BOX)
    cfdata->c_box = devian->container;
  if (devian->conf->source_type == SOURCE_PICTURE)
    cfdata->s_picture = devian->source;
  
  _devian_fill_data(cfdata);
  return cfdata;
}

static void
_devian_free_data(E_Config_Dialog *cfd, dEvianCFData *cfdata)
{
  /* Free the cfdata */
  free(cfdata);
}

/**--APPLY--**/
static int
_devian_basic_apply_data(E_Config_Dialog *cfd, dEvianCFData *cfdata)
{
  /* Actually take our cfdata settings and apply them in real life */
  dEvian *devian;
  devian = cfd->data;

  if (cfdata->c_box)
    {
      if (devian->conf->box_max_size != cfdata->container_box_size)
	{
	  devian->conf->box_max_size = cfdata->container_box_size;
	  devian_container_box_resize_auto(devian->container);
	}
      if (devian->conf->box_anim != cfdata->container_box_animation)
	{
	  devian->conf->box_anim = cfdata->container_box_animation;
	  devian_container_box_animation_start(devian->container, devian->conf->box_anim);
	}
    }

  return 1; /* Apply was OK */
}

static int
_devian_advanced_apply_data(E_Config_Dialog *cfd, dEvianCFData *cfdata)
{
  /* Actually take our cfdata settings and apply them in real life */
  dEvian *devian;
  devian = cfd->data;

  if (cfdata->c_box)
    {
      if (devian->conf->box_max_size != cfdata->container_box_size)
	{
	  DCONTAINER(("CALLING MAX SIZE"));
	  devian->conf->box_max_size = cfdata->container_box_size;
	  devian_container_box_resize_auto(devian->container);
	}
      if (devian->conf->box_auto_resize != cfdata->container_box_auto_resize)
	{
	  DCONTAINER(("CALLING RESIZE AUTO"));
	  devian->conf->box_auto_resize = cfdata->container_box_auto_resize;
	  devian_container_box_resize_auto(devian->container);
	}
      if (devian->conf->box_anim != cfdata->container_box_animation)
	{
	  DCONTAINER(("CALLING ANIM START"));
	  devian->conf->box_anim = cfdata->container_box_animation;
	  devian_container_box_animation_start(devian->container, devian->conf->box_anim);
	}
      devian->conf->box_speed = cfdata->container_box_speed;
    }

  if (cfdata->s_picture)
    {
      if (devian->conf->picture_timer_active != cfdata->source_picture_timer_yn)
	devian_source_picture_timer_change_set(devian->source,
					       cfdata->source_picture_timer_yn,
					       cfdata->source_picture_timer);
      else
	if (devian->conf->picture_timer_s != cfdata->source_picture_timer )
	  devian_source_picture_timer_change_set(devian->source,
						 devian->conf->picture_timer_active,
						 cfdata->source_picture_timer);
    }

  return 1; /* Apply was OK */
}

/**--GUI--**/
static Evas_Object *
_devian_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, dEvianCFData *cfdata)
{
  /* generate the core widget layout for a basic dialog */
  Evas_Object *o, *of, *ob;
  E_Radio_Group *rg;
   
  _devian_fill_data(cfdata);
   
  o = e_widget_table_add(evas, 0);

  if (cfdata->c_box)
    { 
      of = e_widget_frametable_add(evas, _("Box"), 0);
      ob = e_widget_label_add(evas, _("Size"));
      e_widget_frametable_object_append(of, ob,
					0, 0, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_slider_add(evas, 1, 0, _("%1.0f pixels"), 100.0, 500.0, 1.0, 0,
			       NULL, &(cfdata->container_box_size), 100);
      e_widget_frametable_object_append(of, ob,
					1, 0, 1, 1,
					1, 0, 1, 0);
      ob = e_widget_label_add(evas, _("Animation"));
      e_widget_frametable_object_append(of, ob,
					0, 2, 1, 1,
					1, 1, 1, 1);
      rg = e_widget_radio_group_new(&(cfdata->container_box_animation));
      ob = e_widget_radio_add(evas, _("No"), CONTAINER_BOX_ANIM_NO, rg);
      e_widget_frametable_object_append(of, ob,
					1, 2, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_radio_add(evas, _("Line"), CONTAINER_BOX_ANIM_LINE, rg);
      e_widget_frametable_object_append(of, ob,
					1, 3, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_radio_add(evas, _("Gouloum"), CONTAINER_BOX_ANIM_GOULOUM, rg);
      e_widget_frametable_object_append(of, ob,
					1, 4, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_radio_add(evas, _("Ghost"), CONTAINER_BOX_ANIM_GHOST, rg);
      e_widget_frametable_object_append(of, ob,
					1, 5, 1, 1,
					1, 1, 1, 1);
      e_widget_table_object_append(o, of,
				   0, 0, 1, 1,
				   1, 1, 1, 1);
    }

  if(cfdata->s_picture)
    {


    }

  return o;
}

static Evas_Object *
_devian_advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, dEvianCFData *cfdata)
{
  /* generate the core widget layout for an advanced dialog */
  Evas_Object *o, *of, *ob;
  E_Radio_Group *rg;

  _devian_fill_data(cfdata);
   
  o = e_widget_table_add(evas, 0);
   

  if (cfdata->c_box)
    { 
      of = e_widget_frametable_add(evas, _("Box"), 0);
      ob = e_widget_label_add(evas, _("Size"));
      e_widget_frametable_object_append(of, ob,
					0, 0, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_slider_add(evas, 1, 0, _("%1.0f pixels"), 100.0, 500.0, 1.0, 0,
			       NULL, &(cfdata->container_box_size), 100);
      e_widget_frametable_object_append(of, ob,
					1, 0, 1, 1,
					1, 0, 1, 0);
      ob = e_widget_label_add(evas, _("Speed"));
      e_widget_frametable_object_append(of, ob,
					0, 1, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_slider_add(evas, 1, 0, _("%1.0f"), 1.0, 3.0, 1.0, 0,
			       NULL, &(cfdata->container_box_speed), 100);
      e_widget_frametable_object_append(of, ob,
					1, 1, 1, 1,
					1, 0, 1, 0);
      ob = e_widget_label_add(evas, _("Animation"));
      e_widget_frametable_object_append(of, ob,
					0, 2, 1, 1,
					1, 1, 1, 1);
      rg = e_widget_radio_group_new(&(cfdata->container_box_animation));
      ob = e_widget_radio_add(evas, _("No"), CONTAINER_BOX_ANIM_NO, rg);
      e_widget_frametable_object_append(of, ob,
					1, 2, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_radio_add(evas, _("Line"), CONTAINER_BOX_ANIM_LINE, rg);
      e_widget_frametable_object_append(of, ob,
					1, 3, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_radio_add(evas, _("Gouloum"), CONTAINER_BOX_ANIM_GOULOUM, rg);
      e_widget_frametable_object_append(of, ob,
					1, 4, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_radio_add(evas, _("Ghost"), CONTAINER_BOX_ANIM_GHOST, rg);
      e_widget_frametable_object_append(of, ob,
					1, 5, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_check_add(evas, _("Auto Resize"), &(cfdata->container_box_auto_resize));
      e_widget_frametable_object_append(of, ob,
					0, 6, 1, 1,
					1, 1, 1, 1);
      e_widget_table_object_append(o, of,
				   0, 0, 1, 1,
				   1, 1, 1, 1);
    }

  if(cfdata->s_picture)
    {
      of = e_widget_frametable_add(evas, _("Picture"), 0);
      ob = e_widget_check_add(evas, _("Change pictures"), &(cfdata->source_picture_timer_yn));
      e_widget_frametable_object_append(of, ob,
					0, 0, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_label_add(evas, _("Timer"));
      e_widget_frametable_object_append(of, ob,
					0, 1, 1, 1,
					1, 1, 1, 1);
      ob = e_widget_slider_add(evas, 1, 0, _("%1.0fs"), 3.0, 300.0, 5.0, 0,
			       NULL, &(cfdata->source_picture_timer), 200);
      e_widget_frametable_object_append(of, ob,
					0, 2, 1, 1,
					1, 0, 1, 0);
      e_widget_table_object_append(o, of,
				   1, 0, 1, 1,
				   1, 1, 1, 1);
    }

  return o;
}
