#include "dEvian.h"

static void _devian_container_del(dEvian *devian);

/* Public functions */

/*
  Change the current container of a devian
  Return:
    - 0 if ok
    - 1 if fails
*/
int devian_container_change(dEvian *devian, int container)
{
  /* CHECK: if no source, no way */
  if (!devian->source)
    return 1;

  /* CHECK: if already one container, remove it, except if its the one we want */
  if (devian->container)
    _devian_container_del(devian);

  /* Add the new container to the devian */
  switch (container)
    {
    case CONTAINER_BOX:
      {
	if ( devian_container_box_add(devian) )
	  return 1;
	break;
      }

    case CONTAINER_BAR:
      {
	return 1;
      }

    case CONTAINER_NO:
      {
	/* I want no container attached, thank you */
	return 0;
      }

    default : return 1;
    }
  
  return 0;
}

void devian_container_resize_auto(dEvian *devian)
{
  switch (devian->conf->container_type)
    {
    case CONTAINER_BOX:
      {
	devian_container_box_resize_auto(devian->container);
	break;
      }
    case CONTAINER_BAR:
      {
	break;
      }
    }
}

int devian_container_devian_dying(dEvian *devian)
{
  if (devian->conf->container_type == CONTAINER_BOX)
    {
      return devian_container_box_devian_dying(devian->container);
    }
  return 0;
}

int devian_container_edje_part_get(dEvian *devian)
{
  if (!devian->container)
    return -1;

  switch (devian->conf->container_type)
    {
    case CONTAINER_BOX:
      return devian_container_box_edje_part_get(devian->container);
    case CONTAINER_BAR:
      break;
    }

  return -1;
}

int devian_container_edje_part_change(dEvian *devian)
{
  switch (devian->conf->container_type)
    {
    case CONTAINER_BOX:
      return devian_container_box_edje_part_change(devian->container);
    case CONTAINER_BAR:
      break;
    }

  return 0;
}

void devian_container_edje_part_change_set(dEvian *devian, int action)
{
  if (!devian)
    return;
  if (!devian->container)
    return;

  switch (devian->conf->container_type)
    {
    case CONTAINER_BOX:
      devian_container_box_edje_part_change_set(devian->container, action);
      break;
    case CONTAINER_BAR:
      break;
    }

  return;
}

int devian_container_update_name_devian(dEvian *devian)
{
  if (!devian->conf || !devian->container)
    return 0;

  switch (devian->conf->container_type)
    {
    case CONTAINER_BOX:
      return devian_container_box_update_name_devian(devian->container);
    case CONTAINER_BAR:
      return 0;
    }

  return 0;
}

int devian_container_edje_load(Evas_Object *edje_obj, char *part)
{
  int source_w, source_h;

  if (!devian_devian_edje_load(edje_obj, part))
    return 1;

  /* 'Precalculate' (get) borders */
  evas_object_resize(edje_obj, 400, 400);
  edje_object_part_geometry_get(edje_obj, "source0",
				NULL, NULL, &source_w, &source_h);
  dEvianM->theme_box_border_source_w = 400 - source_w;
  dEvianM->theme_box_border_source_h = 400 - source_h;
  DCONTAINER(("Theme borders: x:%d y:%d",
	      dEvianM->theme_box_border_source_w,
	      dEvianM->theme_box_border_source_h));
  evas_object_resize(edje_obj, 0, 0);
  
  return 0;
}

int devian_container_theme_set(char *theme)
{
  dEvian *devian;
  Config_dEvian *conf;
  char *buf;
  int i, max;

  /* Same theme ? */
  if (!strcmp(dEvianM->conf->theme, theme))
    return 0;

  DCONTAINER(("Switching to theme %s", theme));
  
  buf = (char *)evas_stringshare_add(dEvianM->conf->theme);
  evas_stringshare_del(dEvianM->conf->theme);
  dEvianM->conf->theme = (char *)evas_stringshare_add(theme);

  devian_menu_main_menu_refresh();
  
  max = evas_list_count(dEvianM->devians);
  for (i=0; i<max; i++)
    {
      devian = evas_list_nth(dEvianM->devians, i);
      devian_config_devian_save(devian);
      conf = devian_config_devian_new(devian->conf);
      DCONTAINER(("Switch theme devian %d", i));
      devian_devian_del(devian, 0);
      if (!devian_devian_add(conf))
	{
	  DCONTAINER(("Switch theme has failed !"));
	  evas_stringshare_del(dEvianM->conf->theme);
	  dEvianM->conf->theme = buf;
	  devian_devian_add(conf);
	  return 0;
	}
    }
  
  evas_stringshare_del(buf);
  return 1;
}

/* Static functions */

static void _devian_container_del(dEvian *devian)
{
  switch (devian->conf->container_type)
    {
    case CONTAINER_NO: return;
    case CONTAINER_BOX: devian_container_box_del(devian->container);
    case CONTAINER_BAR: return;

    default: return;
    }
}
