#include "dEvian.h"

static int _gadman_init(Container_Box *box);
static void _gadman_shutdown(Container_Box *box);
static void _gadman_change(void *data, E_Gadman_Client *gmc, E_Gadman_Change change);

static void _cb_mouse_down(void *data, Evas* e, Evas_Object* obj,
			   void* event_info);
static void _cb_mouse_in(void *data, Evas* e, Evas_Object* obj,
			 void* event_info);
static void _cb_mouse_out(void *data, Evas* e, Evas_Object* obj,
			  void* event_info);
static void _cb_edje_part_change(void *data, Evas_Object *obj,
				 const char *emission, const char *source);
static void _cb_edje_part_change_start(void *data, Evas_Object *obj,
				       const char *emission, const char *source);
static void _cb_edje_part_change_stop(void *data, Evas_Object *obj,
				      const char *emission, const char *source);
static void _cb_edje_part_change_wanted(void *data, Evas_Object *obj,
				       const char *emission, const char *source);
static void _cb_edje_set_bg(void *data, Evas_Object *obj,
			    const char *emission, const char *source);
static int _cb_ecore_animator(void *data);
static int _cb_timer_anim_ghost(void *data);
static void _animation_stop(Container_Box *box);

/*
  Put the devian in a box
  Return:
   - 0 on success
   - 1 on fail
*/
int devian_container_box_add(dEvian *devian)
{
  Container_Box *box;
  Evas_Object *source0, *source1;

  box = E_NEW(Container_Box, 1);
  
  box->devian = devian;
  box->gmc = NULL;

  DMAIN(("dEvian: init container box START"));

  box->layer = dEvianM->conf->boxs_stacking;

  /* Edje object */
  box->edje = edje_object_add(dEvianM->container->bg_evas);
  if ( devian_container_edje_load(box->edje, "devian/box") )
    {
      devian_container_box_del(box);
      return 1;
    }
  evas_object_layer_set(box->edje, box->layer);
  box->edje_part = 0;
  box->in_transition = 0;

  /* Init show name & number */
  devian_container_box_infos_display_set(box);
  devian_container_box_update_name_devian(box);

  /* Callbacks for mouse events */
  evas_object_event_callback_add(box->edje, EVAS_CALLBACK_MOUSE_DOWN,
				 _cb_mouse_down, box);
  evas_object_event_callback_add(box->edje, EVAS_CALLBACK_MOUSE_IN,
				 _cb_mouse_in, box);
  evas_object_event_callback_add(box->edje, EVAS_CALLBACK_MOUSE_OUT,
				 _cb_mouse_out, box);

  /* Callbacks for controls on edje */
  edje_object_signal_callback_add(box->edje, "pause",
				  "button_playpause",
				  _cb_edje_part_change_stop, box);
  edje_object_signal_callback_add(box->edje, "play",
				  "button_playpause",
				  _cb_edje_part_change_start, box);
  edje_object_signal_callback_add(box->edje, "set_bg",
				  "button_set_bg",
				  _cb_edje_set_bg, box);
  edje_object_signal_callback_add(box->edje, "refresh",
				  "button_refresh",
				  _cb_edje_part_change_wanted, box);

  /* Source displays */
  if ( !devian_source_evas_object_get(devian, &source0, &source1) )
    {
      devian_container_box_del(box);
      return 1;
    }
  /* 0 */
  edje_object_part_swallow(box->edje, "source0", source0);
  /* 1 */
  edje_object_part_swallow(box->edje, "source1", source1);

  /* Clip object */
  /*
  box->clip = evas_object_rectangle_add(dEvianM->container->bg_evas);
  evas_object_color_set(box->clip, 120, 120, 120, 125);
  //evas_object_layer_set(box->clip, box->layer - 1);
  //evas_object_clip_set(source0, box->clip);
  //evas_object_clip_set(source1, box->clip);
  edje_object_part_swallow(box->edje, "evas_source_clip", box->clip);
  */
  
  /* Position & Size */
  box->x = devian->conf->box_x;
  box->y = devian->conf->box_y;
  box->w = 0;
  box->h = 0;
  box->go_w = devian->conf->box_max_size;
  box->go_h = devian->conf->box_max_size;
  evas_object_move(box->edje, box->x, box->y);
  devian_container_box_resize_auto(box);
  //evas_object_resize(box->edje, box->w, box->h);

  /* Show objects, edje and table */
  evas_object_show(source0);
  evas_object_show(source1);
  //evas_object_show(box->clip);
  evas_object_show(box->edje);

  /* Animation */
  box->anim = E_NEW(Container_Box_Anim, 1);
  devian_container_box_animation_start(box, box->devian->conf->box_anim);

  /* Gadman */
  _gadman_init(box);

  /* Attach the object to the devian */
  devian->conf->container_type = CONTAINER_BOX;
  devian->container = box;

  DCONTAINER(("dEvian: init container box OK"));
  
  return 0;
}

void devian_container_box_del(Container_Box *container)
{
  Evas_Object *source0, *source1;

  container->devian->conf->container_type = CONTAINER_NO;
  container->devian->container = NULL;

  /* Gadman */
  _gadman_shutdown(container);

  /* Callbacks for mouse events */
  evas_object_event_callback_del(container->edje, EVAS_CALLBACK_MOUSE_DOWN,
				 _cb_mouse_down);
  evas_object_event_callback_del(container->edje, EVAS_CALLBACK_MOUSE_IN,
				 _cb_mouse_in);
  evas_object_event_callback_del(container->edje, EVAS_CALLBACK_MOUSE_OUT,
				 _cb_mouse_out);

  /* Callbacks for controls */
  edje_object_signal_callback_del(container->edje, "pause",
				  "button_playpause",
				  _cb_edje_part_change_stop);
  edje_object_signal_callback_del(container->edje, "play",
				  "button_playpause",
				  _cb_edje_part_change_start);
  edje_object_signal_callback_del(container->edje, "set",
				  "button_background",
				  _cb_edje_set_bg);
  edje_object_signal_callback_del(container->edje, "refresh",
				  "button_refresh",
				  _cb_edje_part_change_wanted);

  /* Animation */
  if (container->animator)
    {
      if(container == ecore_animator_del(container->animator))
	{
	  DCONTAINER(("Ecore animator deletion ok"));
	  container->animator = NULL;
	}
    }
  devian_container_box_animation_start(container, CONTAINER_BOX_ANIM_NO);
  //_animation_stop(container);
  E_FREE(container->anim);

  /* Source */
  if ( devian_source_evas_object_get(container->devian, &source0, &source1) )
    {
      evas_object_hide(source0);
      evas_object_hide(source1);
    }

  /* Clip object */
  /*
  if (container->clip)
    {
      evas_object_hide(container->clip);
      evas_object_del(container->clip);
    }
  */

  /* Edje object */
  if (container->edje)
    {
      evas_object_hide(container->edje);
      evas_object_del(container->edje);
    }
  
  dEvianM->container_box_count--;

  E_FREE(container);
}

void devian_container_box_resize_auto(Container_Box *box)
{
  int w, h;
  double w2, h2;
  int max_w, max_h;
  double thm_w, thm_h;

  /* Calc the size */
  if (!box->devian->dying)
    {
      max_w = box->devian->conf->box_max_size;
      max_h = box->devian->conf->box_max_size;
      thm_h = dEvianM->theme_box_border_source_h;
      thm_w = dEvianM->theme_box_border_source_w;

      if (box->devian->conf->box_auto_resize)
	{
	  double ratio;
	  if (!devian_source_original_size_get(box->devian, box->edje_part,
					       &w, &h))
	    {
	      printf("Cant get sources size !!!\n");
	      return;
	    }
	  DCONTAINER((" - Auto resize - (im %dx%d) (max %dx%d) (r %f %f)", w, h, max_w, max_h, thm_w, thm_h));

	  /* Box size limited to max box size, keeping ratio */
	  /* Size of edje <-> size of the source
	     What size the box should have to perfectly fit the source */
	  /*
	  //if (w > max_w)
	  {
	  ratio = (float)h / (float)w;
	  w = max_w + thm_w;
	  h = (int)((float)w * ratio);
	  }
	  if (h > max_h + thm_h)
	  {
	  ratio = (float)w / (float)h;
	  h = max_h + thm_h;
	  w = (int)((float)h * ratio);
	  }
	  */
	  w2 = (double)w;
	  h2 = (double)h;
      
	  {
	    ratio = h2 / w2;
	    w2 = (double)max_w;
	    h2 = w2 * ratio;
	  }
	  if ((int)h2 > max_h)
	    {
	      ratio = w2 / h2;
	      h2 = (double)max_h;
	      w2 = h2 * ratio;
	    }

	  w = (int)w2 + thm_w;
	  h = (int)h2 + thm_h;
	}
      else
	{
	  DCONTAINER((" - NO Auto resize -"));
	  w = max_w;
	  h = max_h;
	}
      DCONTAINER(("Box new size : %dx%d", w, h));

      if ((box->w == w) && (box->h == h))
	return;

      box->go_w = w;
      box->go_h = h;
    }

  /* Set the size */
  if (dEvianM->conf->boxs_nice_resize)
    {
      /* Nice resize */
      box->in_resize = 1;
      if (box->gmc)
	{
	  e_gadman_client_aspect_set(box->gmc,
				     ((float)box->go_w/(float)box->go_h),
				     ((float)box->go_w/(float)box->go_h));
	}
      if (!box->animator)
	{
	  box->animator = ecore_animator_add(_cb_ecore_animator, box);
	}
    }
  else
    {
      /* Quick resize */
      if(((box->go_w == 0) || (box->go_h == 0)) && box->devian->dying)
        {
	  devian_devian_del(box->devian, 1);
	  return;
        }
      if (box->gmc)
	{
	  e_gadman_client_aspect_set(box->gmc,
				     ((float)box->go_w/(float)box->go_h),
				     ((float)box->go_w/(float)box->go_h));
	  if(e_gadman_mode_get(box->gmc->gadman) != E_GADMAN_MODE_NORMAL)
	    return;
	}
      box->w = box->go_w;
      box->h = box->go_h;
      if (box->gmc)
	{
	  box->gmc->w = box->w;
	  box->gmc->h = box->h;
	}
      evas_object_resize(box->edje, box->w, box->h);
    }

  return;
}

int devian_container_box_devian_dying(Container_Box *box)
{
  box->go_w = 0;
  box->go_h = 0;
  devian_container_box_resize_auto(box);
  return 1;
}

int devian_container_box_edje_part_get(Container_Box *container)
{
  return container->edje_part;
}

int devian_container_box_edje_part_change(Container_Box *container)
{
  Evas_Object *source0, *source1;
  char *source_name;

  devian_source_evas_object_get(container->devian,
					&source0, &source1);

  if (!source0 || !source1)
    return 0;

  if (container->in_transition)
    return 0;

  /* Change the name displayed */
  source_name = NULL;
  source_name = devian_source_name_get(container->devian, !container->edje_part);
  if (source_name)
    {
            edje_object_part_text_set(container->edje, "name", source_name);
    }
  else
    {
            edje_object_part_text_set(container->edje, "name", "???");
    }

  /* Start transition */
  if(!container->edje_part)
    {
      container->edje_part = 1;
      edje_object_part_swallow(container->edje, "source1", source1);
      evas_object_show(source1);
      container->in_transition = 1;
      if (dEvianM->conf->boxs_nice_trans)
	{
	  edje_object_signal_emit(container->edje, "source_transition_0-1_go", "");
	  edje_object_signal_callback_add(container->edje, "source_transition_0-1_end",
					  "th",
					  _cb_edje_part_change, container);
	}
      else
	{
	  edje_object_signal_emit(container->edje, "source_transition_q_0-1_go", "");
	  edje_object_signal_callback_add(container->edje, "source_transition_q_0-1_end",
					  "th",
					  _cb_edje_part_change, container);
	}
    }
  else
    {
      container->edje_part = 0;
      edje_object_part_swallow(container->edje, "source0", source0);
      evas_object_show(source0);
      container->in_transition = 0;
      if (dEvianM->conf->boxs_nice_trans)
	{
	  edje_object_signal_emit(container->edje, "source_transition_1-0_go", "");
	  edje_object_signal_callback_add(container->edje, "source_transition_1-0_end",
					  "th",
					  _cb_edje_part_change, container);
	}
      else
	{
	  edje_object_signal_emit(container->edje, "source_transition_q_1-0_go", "");
	  edje_object_signal_callback_add(container->edje, "source_transition_q_1-0_end",
					  "th",
					  _cb_edje_part_change, container);
	}
    }

  DCONTAINER(("Begining of transition"));
  return 1;
}

void devian_container_box_edje_part_change_set(Container_Box *box, int action)
{
  DCONTAINER(("Message part change set send (%d)", action));
  edje_object_message_send(box->edje, EDJE_MESSAGE_INT,
			   DEVIAN_CONTAINER_BOX_EDJE_MSG_PART_CHANGE_SET, &action);
}

int devian_container_box_update_name_devian(Container_Box *box)
{
  if (!box->devian->name)
    return 0;

  edje_object_part_text_set(box->edje, "number", box->devian->name);

  return 1;
}

void devian_container_box_infos_display_set(Container_Box *box)
{
  if (dEvianM->conf->boxs_always_number)
    edje_object_signal_emit(box->edje, "number_always_activate", "");
  else
    edje_object_signal_emit(box->edje, "number_always_deactivate", "");
  if (dEvianM->conf->boxs_always_infos)
    edje_object_signal_emit(box->edje, "name_always_activate", "");
  else
    edje_object_signal_emit(box->edje, "name_always_deactivate", "");
}

/*
  Start a new animation of on a box
  Stop all other anim / restart current anim
  Args:
   - box
   - number of the anim
  Returns:
   - 1 if the animation has been set
   - 0 otherwise
*/
int devian_container_box_animation_start(Container_Box *box, int anim_num)
{
  _animation_stop(box);

  switch (anim_num)
    {

    case CONTAINER_BOX_ANIM_NO:
      {
	evas_object_color_set(box->edje, 255, 255, 255, 255);
	break;
      }

    case CONTAINER_BOX_ANIM_LINE:
      {
	evas_object_color_set(box->edje, 255, 255, 255, 255);
	break;
      }

    case CONTAINER_BOX_ANIM_GOULOUM:
      {
	evas_object_color_set(box->edje, 255, 255, 255, 255);
	box->anim->data[0] = 0;
	box->anim->data[1] = 1;
	box->anim->data[2] = 1;
	break;
      }

    case CONTAINER_BOX_ANIM_GHOST:
      {
	/* In fade */
	box->anim->data[1] = 0;
	/* 0: fade->trans, 1: fade->visible */
	box->anim->data[2] = 1;
	/* Alpha */
	evas_object_color_set(box->edje, 255, 255, 255, 255);
	box->anim->data[3] = 255;
	/* Timer */
	if (box->anim->timer)
	  ecore_timer_del(box->anim->timer);
	box->anim->timer = ecore_timer_add(dEvianM->conf->boxs_anim_ghost_timer,
					   _cb_timer_anim_ghost,
					   box);
	break;
      }

    default:
      {
	return 0;
      }

    }

  DCONTAINER(("Start animation %d", anim_num));

  box->devian->conf->box_anim = anim_num;
  
  if (anim_num)
    {
      if (!box->animator &&
	  (anim_num != CONTAINER_BOX_ANIM_GHOST))
	box->animator = ecore_animator_add(_cb_ecore_animator, box);
      if (anim_num != CONTAINER_BOX_ANIM_GHOST)
	{
	  ecore_timer_del(box->anim->timer);
	  box->anim->timer = NULL;
	}
    }
  else
    {
      if (box->anim->timer)
	{
	  ecore_timer_del(box->anim->timer);
	  box->anim->timer = NULL;
	}
    }

  return 1;
}

void devian_container_box_random_pos_get(int *x, int *y, int max_size)
{
  *x = rand()%(dEvianM->canvas_w - (max_size + dEvianM->theme_box_border_source_w));
  *y = rand()%(dEvianM->canvas_h - (max_size + dEvianM->theme_box_border_source_h));
}

/* PRIVATE FUNCTIONS */

static int _gadman_init(Container_Box *box)
{
  box->gmc = e_gadman_client_new(dEvianM->container->gadman);

  box->gmc_init = 1;

  e_gadman_client_domain_set(box->gmc, "module.dEvian", dEvianM->container_box_count++);
  e_gadman_client_load(box->gmc); /* Only to avoid segfault when del the gmc */

  e_gadman_client_zone_set(box->gmc, e_zone_current_get(dEvianM->container));
  e_gadman_client_policy_set(box->gmc,
			     E_GADMAN_POLICY_ANYWHERE |
			     E_GADMAN_POLICY_HMOVE |
			     E_GADMAN_POLICY_VMOVE |
			     E_GADMAN_POLICY_HSIZE |
			     E_GADMAN_POLICY_VSIZE);
  e_gadman_client_min_size_set(box->gmc, 100, 100);
  e_gadman_client_max_size_set(box->gmc, 500, 500);
  e_gadman_client_auto_size_set(box->gmc,
				(box->w),
				(box->h));
  //e_gadman_client_align_set(box->gmc,
  //		    ((float)box->w/(float)box->h),
  //		    ((float)box->w/(float)box->h));
  e_gadman_client_aspect_set(box->gmc,
			     ((float)box->w/(float)box->h),
			     ((float)box->w/(float)box->h));
  e_gadman_client_padding_set(box->gmc,
			      10, 10,
			      10, 10);
  e_gadman_client_resize(box->gmc, box->w, box->h);
  e_gadman_client_change_func_set(box->gmc, _gadman_change, box);
  box->gmc->x = box->x;
  box->gmc->y = box->y;

  //e_gadman_client_load(box->gmc);

  return 1;
}

static void _gadman_shutdown(Container_Box *box)
{
  if (!box->gmc)
    return;

  while ( e_object_unref(E_OBJECT(box->gmc)) > 0 );
}

static void _gadman_change(void *data, E_Gadman_Client *gmc, E_Gadman_Change change)
{
  Container_Box *box;
  Evas_Coord x, y, w, h;

  box = data;
 
  switch (change)
    {

    case E_GADMAN_CHANGE_MOVE_RESIZE:
      if (box->gmc_init)
	{
	  box->gmc_init = 0;
	  break;
	}
      
      e_gadman_client_geometry_get(box->gmc, &x, &y, &w, &h);
      box->x = x;
      box->y = y;
      box->w = w;
      box->h = h;

      box->go_w = box->w;
      box->go_h = box->h;
      if (w>h)
	box->devian->conf->box_max_size = box->w;
      else
	box->devian->conf->box_max_size = box->h;
      box->in_resize = 0;

      evas_object_move(box->edje, x, y);
      evas_object_resize(box->edje, w, h);
      break;

    case E_GADMAN_CHANGE_RAISE:
      evas_object_raise(box->edje);
      break;

    default:
      break;

    }

}

static void _cb_mouse_down(void *data, Evas* e, Evas_Object* obj,
			   void* event_info)
{
  Evas_Event_Mouse_Down* ev;
  Container_Box *box;

  box = data;
  ev = event_info;
  
  if (ev->button == 2)
    {
      box->layer = CONTAINER_BOX_STACKING_UNDER;
      evas_object_layer_set(box->edje, CONTAINER_BOX_STACKING_UNDER);
    }
  else
    {
      if (ev->button == 3)
	{ 
	  e_menu_activate_mouse(box->devian->menu,
				e_zone_current_get(dEvianM->container),
				ev->output.x, ev->output.y,
				1, 1, E_MENU_POP_DIRECTION_AUTO,
				ev->timestamp);
	  e_util_container_fake_mouse_up_all_later(dEvianM->container);
	}
    }
}

static void _cb_mouse_in(void *data, Evas* e, Evas_Object* obj,
			 void* event_info)
{
  Container_Box *box;

  box = data;
  
  if (box->layer < CONTAINER_BOX_STACKING_UPPER)
    { 
      box->layer = CONTAINER_BOX_STACKING_UPPER;
      DCONTAINER(("dEvian: mouse in"));
      evas_object_layer_set(box->edje, CONTAINER_BOX_STACKING_UPPER);
    }
}

static void _cb_mouse_out(void *data, Evas* e, Evas_Object* obj,
			 void* event_info)
{
  Container_Box *box;

  box = data;
  
  if ((box->layer != CONTAINER_BOX_STACKING_NORMAL))
    { 
      box->layer = CONTAINER_BOX_STACKING_NORMAL;
      DCONTAINER(("dEvian: mouse out"));
      evas_object_layer_set(box->edje, CONTAINER_BOX_STACKING_NORMAL);
    }
}

static void _cb_edje_part_change(void *data, Evas_Object *obj,
				 const char *emission, const char *source)
{
  Container_Box *container;
  dEvian *devian;
  Evas_Object *source0, *source1;

  DCONTAINER(("End of transition has begin"));

  container = data;
  devian = container->devian;

  if (!container->edje || !obj)
    {
      DCONTAINER(("NO EDJE !!!"));
      container->in_transition = 0;
      return;
    }

  devian_source_evas_object_get(devian,
					&source0, &source1);

  if(!strcmp(emission, "source_transition_0-1_end"))
    {
      evas_object_hide(source0);
      edje_object_part_unswallow(container->edje, source0);
      edje_object_signal_callback_del(obj, "source_transition_0-1_end",
				      "th",
				      _cb_edje_part_change);
      container->in_transition = 0;
      devian_source_detach(devian, 0);
    }
  else
    {
      if(!strcmp(emission, "source_transition_q_0-1_end"))
	{ 
	  evas_object_hide(source0);
	  edje_object_part_unswallow(container->edje, source0);
	  edje_object_signal_callback_del(obj, "source_transition_q_0-1_end",
					  "th",
					  _cb_edje_part_change);
	  container->in_transition = 0;
	  devian_source_detach(devian, 0);
	}
      else
	{
	  if(!strcmp(emission, "source_transition_1-0_end"))
	    {
	      evas_object_hide(source1);
	      edje_object_part_unswallow(container->edje, source1);
	      edje_object_signal_callback_del(obj, "source_transition_1-0_end",
					      "th",
					      _cb_edje_part_change);
	      container->in_transition = 0;
	      devian_source_detach(devian, 1);
	    }
	  else
	    {
	      if(!strcmp(emission, "source_transition_q_1-0_end"))
		{
		  evas_object_hide(source1);
		  edje_object_part_unswallow(container->edje, source1);
		  edje_object_signal_callback_del(obj, "source_transition_q_1-0_end",
						  "th",
						  _cb_edje_part_change);
		  container->in_transition = 0;
		  devian_source_detach(devian, 1);
		}
	      else
		printf("dEvian: Get unknow edje transition signal !\n");
	    }
	}
    }
  
  DCONTAINER(("End of transition"));
}

static void _cb_edje_part_change_start(void *data, Evas_Object *obj,
				       const char *emission, const char *source)
{
  Container_Box *box;
  box = (Container_Box *)data;

  DCONTAINER(("Edje cb part_change_start"));
  devian_source_timer_change_set(box->devian, 1, 0);
}

static void _cb_edje_part_change_stop(void *data, Evas_Object *obj,
				      const char *emission, const char *source)
{
  Container_Box *box;
  box = (Container_Box *)data;

  DCONTAINER(("Edje cb part_change_stop"));
  devian_source_timer_change_set(box->devian, 0, 0);
}

static void _cb_edje_part_change_wanted(void *data, Evas_Object *obj,
				       const char *emission, const char *source)
{
  int i;
  Container_Box *box;
  box = (Container_Box *)data;

  DCONTAINER(("Edje cb part_change_wanted"));
  i = devian_source_refresh(box->devian);
  DCONTAINER(("Edje cb: transition returns %d", i));
}

static void _cb_edje_set_bg(void *data, Evas_Object *obj,
			    const char *emission, const char *source)
{
  Container_Box *box;
  box = (Container_Box *)data;

  DCONTAINER(("Edje cb set_bg"));
  devian_source_set_bg(box->devian);
}

/*
  The animator for one Container_Box
*/
static int _cb_ecore_animator(void *data)
{
  Container_Box *box;
  int spd;

  box = data;
  spd = box->devian->conf->box_speed;

  if(e_gadman_mode_get(box->gmc->gadman) == E_GADMAN_MODE_NORMAL)
    {
  /* Resize */
  if (box->in_resize)
    {
      int diff_w, diff_h;
      diff_w = box->w - box->go_w;
      diff_h = box->h - box->go_h;
      
      /* CHECK: End of resize ? */
      if((diff_w == 0) && (diff_h == 0))
	{
	  DCONTAINER(("Resize finished"));
	  box->in_resize = 0;
	  if(box->devian->dying)
	  {
	  	devian_devian_del(box->devian, 1);
	  	return 0;
	  }
	}
      else
	{
	  if(diff_w > 0)
	    box->w = box->w - 1;
	  else
	    if(diff_w < 0)
	      box->w = box->w + 1;
	  if(diff_h > 0)
	    box->h = box->h - 1;
	  else
	    if(diff_h < 0)
	      box->h = box->h + 1;

	  box->gmc->w = box->w;
	  box->gmc->h = box->h;
	  evas_object_resize(box->edje, box->w, box->h);
	  
	}
    }

  /* Animation */
  switch (box->devian->conf->box_anim)
    {
    case CONTAINER_BOX_ANIM_NO:
      break;
    case CONTAINER_BOX_ANIM_LINE:
      {
	box->x = box->x + spd;
	if ( (box->x + box->w) > dEvianM->canvas_w )
	  box->x = 0;

	box->gmc->x = box->x;
	box->gmc->y = box->y;
	evas_object_move(box->edje, box->x, box->y);

	break;
      }
    case CONTAINER_BOX_ANIM_GOULOUM:
      {
	box->x = box->x + box->anim->data[1]*spd;
	box->y = box->y + box->anim->data[2]*spd;
	if ( (box->x + box->w) > dEvianM->canvas_w )
	  box->anim->data[1] = -1;
	if ( (box->x) < 0 )
	  box->anim->data[1] = 1;
	if ( (box->y + box->h) > dEvianM->canvas_h )
	  box->anim->data[2] = -1;
	if ( (box->y) < 0 )
	  box->anim->data[2] = 1;

	box->gmc->x = box->x;
	box->gmc->y = box->y;
	evas_object_move(box->edje, box->x, box->y);

	break;
      }
    case CONTAINER_BOX_ANIM_GHOST:
      {
	if (box->anim->data[1])
	  {
	    if (!box->anim->data[2])
	      {
		box->anim->data[3] = box->anim->data[3] - 10;
		if (box->anim->data[3] < 0)
		  box->anim->data[3] = 0;
		evas_object_color_set(box->edje, 255, 255, 255,
				      box->anim->data[3]);
		if (box->anim->data[3] == 0)
		  {
		    box->anim->data[2] = 1;
		    devian_container_box_random_pos_get(&box->x, &box->y,
							box->devian->conf->box_max_size);
		    box->gmc->x = box->x;
		    box->gmc->y = box->y;
		    evas_object_move(box->edje, box->x, box->y);
		  }
	      }
	    else
	      {
		box->anim->data[3] = box->anim->data[3] + 10;
		if (box->anim->data[3] > 255)
		  box->anim->data[3] = 255;
		evas_object_color_set(box->edje, 255, 255, 255,
				      box->anim->data[3]);
		if (box->anim->data[3] == 255)
		  {
		    box->anim->data[1] = 0;
		  }
	      }
	  }
	break;
      }
    default: box->devian->conf->box_anim = CONTAINER_BOX_ANIM_NO;
    }

    }
  
  /* If there are no more animations for box , destroy ecore animator */
  if (!box->in_resize &&
      (!box->devian->conf->box_anim ||
       (box->devian->conf->box_anim==CONTAINER_BOX_ANIM_GHOST &&
	!box->anim->data[1])))
    {
      DCONTAINER(("> Stop anim callback <"));
      box->animator = NULL;
      return 0;
    }
  else
    return 1;
}

static int _cb_timer_anim_ghost(void *data)
{
  Container_Box *box;
  box = (Container_Box *)data;

  if (!box->animator)
    box->animator = ecore_animator_add(_cb_ecore_animator, box);

  box->anim->data[1] = 1;
  box->anim->data[2] = !box->anim->data[2];

  return 1;
}

static void _animation_stop(Container_Box *box)
{
  if (!box->anim)
    return;
  box->devian->conf->box_anim = CONTAINER_BOX_ANIM_NO;
}
