#include "dEvian.h"


int devian_devian_main_init(E_Module *m)
{
  Config_dEvian *cfg_devian;
  int i;

  dEvianM = E_NEW(dEvian_Main, 1);

  dEvianM->container = e_container_current_get(e_manager_current_get());
  evas_output_viewport_get(dEvianM->container->bg_evas, NULL, NULL, &dEvianM->canvas_w, &dEvianM->canvas_h);
  DMAIN(("dEvian : Canvas %d x %d",dEvianM->canvas_w,dEvianM->canvas_h));

  dEvianM->module = m;

  dEvianM->devians = NULL;
  dEvianM->themes_menu = NULL;
  dEvianM->container_box_count = 0;

  dEvianM->display = devian_display_init();

  dEvianM->conf = devian_config_init();
  if (!dEvianM->conf)
      return 0;

  devian_menu_main_menu_init();
  if (!dEvianM->main_menu)
    return 0;

  if ( devian_data_picture_list_local_init() )
    {
      fprintf(stderr, "dEvian: can't init picture list (local) !\n");
      return 0;
    }

  if ( !devian_data_picture_cache_init() )
    {
      fprintf(stderr, "dEvian: can't init picture cache !\n");
      return 0;
    }  

  /* Init devians like it is in the config file */
  DMAIN(("%d devians to create", dEvianM->conf->nb_devian));
  for (i=0; i<dEvianM->conf->nb_devian; i++)
    {
      cfg_devian = evas_list_nth(dEvianM->conf->devians_conf, i);
      if (!devian_devian_add(cfg_devian))
	{
	  fprintf(stderr, "dEvian: Not good, a devian couldnt be created\n");
	  evas_list_remove (dEvianM->conf->devians_conf,
			    cfg_devian);
	}
    }
  
  DMAIN(("dEvian : init_devian OK\n"));

  return 1;
}


void devian_devian_main_shutdown(void)
{
  DMAIN(("Main shutdown begins"));

  devian_devian_del_all();

  devian_data_picture_cache_shutdown();
  devian_data_picture_list_local_shutdown();

  devian_menu_main_menu_shutdown();
  devian_config_dialog_main_shutdown();
  devian_config_free();
  devian_display_shutdown();

  E_FREE(dEvianM);
}

/*
  Creates a new dEvian and add it to the devians list
  Args:
   - devian_main
   - Pointer to the config to use for new devian.
     If its NULL, we create a new config
  Return:
   Pointer to the new dEvian or NULL if fails
*/
dEvian *devian_devian_add(Config_dEvian *cfg_devian)
{
  dEvian *devian;

  devian = E_NEW(dEvian, 1);
  devian->menu = NULL;
  devian->name = NULL;
  devian->source = NULL;
  devian->container = NULL;
  devian->dying = 0;
  devian->dialog_conf = NULL;
  devian->conf = NULL;
  devian->menu = NULL;

  /* Append new devian to the list of devians */
  dEvianM->devians = evas_list_append (dEvianM->devians,
				       devian);

  /* Name */
  devian_devian_set_name(devian, SOURCE_NO, NULL);

  /* Config */
  if (cfg_devian)
    {
      /* Use existing config */
      devian->conf = cfg_devian;
    }
  else
    {
      /* New config */
      devian->conf = devian_config_devian_new(NULL);
      if (!devian->conf)
	{
	  devian_devian_del(devian, 1);
	  return NULL;
	}
    }
  
  /* Menu */
  devian->menu = devian_menu_init(devian);
  if (!devian->menu)
    {
      devian_devian_del(devian, 1);
      return NULL;
    }

  /* Source */
  if ( devian_source_change(devian, devian->conf->source_type) )
    {
      devian_devian_del(devian, 1);
      return NULL;
    }

  /* Container */
  if ( devian_container_change(devian, devian->conf->container_type) )
    {
      devian_devian_del(devian, 1);
      return NULL;
    }

  DMAIN(("dEvian: New devian OK"));

  return devian;
}

void devian_devian_del(dEvian *devian, int now)
{
  if (!now)
    {
      devian->dying = 1;
      if (devian_container_devian_dying(devian))
	return;
    }
	
  DMAIN(("dEvian: del devian container"));
  devian_container_change(devian, CONTAINER_NO);
  DMAIN(("dEvian: del devian source"));
  devian_source_change(devian, SOURCE_NO);
  DMAIN(("dEvian: Free devian menu"));
  devian_menu_shutdown(devian);
  DMAIN(("dEvian: Free devian config dialog"));
  devian_config_dialog_devian_shutdown(devian);
  DMAIN(("dEvian: Free devian config"));
  devian_config_devian_free(devian);
  DMAIN(("dEvian: Free devian remove"));
  dEvianM->devians = evas_list_remove (dEvianM->devians,
					   devian);
  DMAIN(("Rename dEvians"));
  devian_devian_set_name(NULL, 0, NULL);
  DMAIN(("dEvian: Free devian refresh main menu"));
  devian_menu_main_menu_refresh();
  DMAIN(("dEvian: Free devian free"));

  if (devian->name)
    E_FREE(devian->name);
  E_FREE(devian);

  DMAIN(("dEvian: Free devian OK"));
}

void devian_devian_del_all(void)
{
  while (dEvianM->devians)
    {
      DMAIN(("dEvian: devian_del"));
      devian_devian_del(evas_list_data(dEvianM->devians), 1);
    }
}

/*
  Set the name of one / all devian(s)
  Args:
   - If devian isnt NULL, set the name only for the pointed devian.
   The name is choosen with -source, and you can add -extra
   - Source: type of the source of the devian indicated as -devian
   - Extra: you can add extra one / all devian(s) name's, or set it to null
*/
void devian_devian_set_name(dEvian *devian, int source, char *extra)
{
  char *name;
  int n;

  if (!devian)
    {
      Evas_List *l;
      dEvian *d;

      n=1;
      for (l=dEvianM->devians; l; l=evas_list_next(l))
	{
	  name = E_NEW(char, DEVIAN_NAME_LEN);
	  d = evas_list_data(l);
	  if (d->name && d->conf)
	    {
	      E_FREE(d->name);
	      switch (d->conf->source_type)
		{
		case SOURCE_PICTURE:
		  snprintf(name, DEVIAN_NAME_LEN, "Picture %d", n);
		  break;
		case SOURCE_NO:
		  snprintf(name, DEVIAN_NAME_LEN, "No source %d", n);
		  break;
		}
	    }
	  else
	    {
	      if (d->name)
		E_FREE(d->name);
	      snprintf(name, DEVIAN_NAME_LEN, "Invalid devian %d", n);
	    }
	  d->name = name;
	  devian_container_update_name_devian(d);
	  n++;
	}
      devian_menu_main_menu_refresh();
    }
  else
    {
      name = E_NEW(char, DEVIAN_NAME_LEN);
      n = evas_list_count(dEvianM->devians);
      switch (source)
	{
	case SOURCE_PICTURE:
	  if (extra)
	    snprintf(name, DEVIAN_NAME_LEN, "Picture %d %s", n, extra);
	  else
	  snprintf(name, DEVIAN_NAME_LEN, "Picture %d", n);
	  break;
	case SOURCE_NO:
	  if (extra)
	    snprintf(name, DEVIAN_NAME_LEN, "No source %d %s", n, extra);
	  else
	    snprintf(name, DEVIAN_NAME_LEN, "No source %d", n);
	  break;
	default: 
	  if (extra)
	    snprintf(name, DEVIAN_NAME_LEN, "Invalid dEvian %d %s", n, extra);
	  else
	    snprintf(name, DEVIAN_NAME_LEN, "Invalid dEvian %d", n);
	  break;
	}
      if (devian->name)
	E_FREE(devian->name);
      devian->name = name;
      if (devian->container)
	devian_container_update_name_devian(devian);
      if (devian->menu)
	devian_menu_main_menu_refresh();
    }

  return;
}

char *devian_display_init(void)
{
  char *display = NULL;
  char *tmp;

  tmp = getenv("DISPLAY");
  if (tmp) display = strdup(tmp);

  /* make sure the display var is of the form name:0.0 or :0.0 */
  if (display)
    {
      char *p;
      char buf[1024];

      p = strrchr(display, ':');
      if (!p)
	{
	  snprintf(buf, sizeof(buf), "%s:0.0", display);
	  free(display);
	  display = strdup(buf);
	}
      else
	{
	  p = strrchr(p, '.');
	  if (!p)
	    {
	      snprintf(buf, sizeof(buf), "%s.0", display);
	      free(display);
	      display = strdup(buf);
	    }
	}
    }
  else
    display = strdup(":0.0");

  /* Init E Lib */
  if (display)
    e_lib_init(display);
  else
    {
      display = strdup(":0.0");
      e_lib_init(display);
    }

  return display;
}

void devian_display_shutdown(void)
{
  free(dEvianM->display);
  dEvianM->display = NULL;
  e_lib_shutdown();
}

int devian_devian_edje_load(Evas_Object *edje_obj, char *part)
{
  char path[DEVIAN_MAX_PATH];

  snprintf(path, DEVIAN_MAX_PATH, "%s/%s/%s%s",
           e_module_dir_get(dEvianM->module), "themes",
           dEvianM->conf->theme,
           ".edj");
  if ( !edje_object_file_set(edje_obj, path, part) )
    {
      char buf[4096];
      snprintf(buf, sizeof(buf),
               _(
                 "<hilight>Error !</hilight><br>"
                 "Did not find <hilight>theme</hilight> %s group %s"),
               path, part);
      e_module_dialog_show(_("dEvian Module Error"), buf);
      return 0;
    }
#ifdef DEBUG_MAIN
  else
    {
      fprintf(stderr, "Edje loaded %s group %s\n" , path, part);
    }
#endif
  
  return 1;
}
