#include "dEvian.h"

static void _devian_main_menu_cb_add_devian(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_main_menu_cb_theme_change(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_main_menu_cb_configure(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_main_menu_cb_delete_all_devians(void *data, E_Menu *m, E_Menu_Item *mi);
static E_Menu *_menu_theme_generate(void);

static void _devian_menu_cb_source_infos(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_menu_cb_source_viewer(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_menu_cb_devian_delete(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_menu_cb_configure(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_menu_cb_set_container_box(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_menu_cb_set_source_picture(void *data, E_Menu *m, E_Menu_Item *mi);
static void _devian_menu_cb_edit_mode(void *data, E_Menu *m, E_Menu_Item *mi);

/* Main menu section */

static void _devian_main_menu_cb_add_devian(void *data, E_Menu *m, E_Menu_Item *mi)
{
  devian_devian_add(NULL);
}

static void _devian_main_menu_cb_theme_change(void *data, E_Menu *m, E_Menu_Item *mi)
{
  if (!mi->label)
    return;

  devian_container_theme_set(mi->label);
}

static void _devian_main_menu_cb_configure(void *data, E_Menu *m, E_Menu_Item *mi)
{
  dEvianM->dialog_conf = devian_config_dialog_main(dEvianM->container);
  e_object_ref(E_OBJECT(dEvianM->dialog_conf));
}

static void _devian_main_menu_cb_delete_all_devians(void *data, E_Menu *m, E_Menu_Item *mi)
{
  devian_devian_del_all();
}

static E_Menu *_menu_theme_generate(void)
{
  char *file, *ext;
  char name[DEVIAN_MAX_PATH];
  char themes_path[DEVIAN_MAX_PATH];
  int name_l;
  E_Menu *mn;
  E_Menu_Item *mi;
  Ecore_List *list = NULL;

  if (dEvianM->themes_menu)
    {
      e_menu_deactivate(dEvianM->themes_menu);
      e_object_del(E_OBJECT(dEvianM->themes_menu));
    }
  mn = e_menu_new();
  dEvianM->themes_menu = mn;

  snprintf(themes_path, DEVIAN_MAX_PATH, "%s%s", e_module_dir_get(dEvianM->module), "/themes");

  if (!ecore_file_is_dir(themes_path))
    return NULL;

  list = ecore_file_ls(themes_path);
  if (ecore_list_is_empty(list) || !list)
    {
      char buf[4096];
      snprintf(buf, sizeof(buf),
               _(
                 "<hilight>Directory %s is empty or doesnt exists.</hilight><br><br>"
		 "No themes found !!!"
		 ),
               themes_path
               );
      e_module_dialog_show(_("dEvian Module Error"), buf);
      return NULL;
    }
  
  while ((file = (char*)ecore_list_next(list)))
    {
      ext = strrchr(file, '.');
      if (!ext)
	continue;
      name_l = strlen(file) - strlen(ext);
      if (name_l<=0)
	continue;
      name[name_l] = '\0';
      strncpy(name, file, name_l);

      mi = e_menu_item_new(mn);
      e_menu_item_label_set(mi, name);
      e_menu_item_radio_set(mi, 1);
      e_menu_item_radio_group_set(mi, 1);
      if ( !strcmp(dEvianM->conf->theme, name) )
	e_menu_item_toggle_set(mi, 1);
      e_menu_item_callback_set(mi, _devian_main_menu_cb_theme_change, NULL);
    }

  return mn;
}

E_Menu *devian_menu_main_menu_init(void)
{
  E_Menu_Item *mi;
  E_Menu *mn;

  DMAIN(("dEvian : main menu"));
  
  mn = e_menu_new();

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Add dEvian"));
  e_menu_item_callback_set(mi, _devian_main_menu_cb_add_devian, NULL);

  mi = e_menu_item_new(mn);
  e_menu_item_separator_set(mi, 1);

  if ( !(dEvianM->themes_menu = _menu_theme_generate()) )
    return NULL;

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Theme"));
  e_menu_item_submenu_set(mi, dEvianM->themes_menu);

  mi = e_menu_item_new(mn);
  e_menu_item_separator_set(mi, 1);
  
  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Configure Main"));
  e_menu_item_callback_set(mi, _devian_main_menu_cb_configure, NULL);

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Delete all dEvians"));
  e_menu_item_callback_set(mi, _devian_main_menu_cb_delete_all_devians, NULL);

  dEvianM->main_menu = mn;

  DMAIN(("dEvian : main menu fin"));

  return mn;
}

void devian_menu_main_menu_shutdown(void)
{
  if (dEvianM->main_menu)
    {
      e_menu_deactivate(dEvianM->main_menu);
      e_object_del(E_OBJECT(dEvianM->main_menu));
      dEvianM->main_menu = NULL;
    }
}

void devian_menu_main_menu_refresh(void)
{
  devian_menu_main_menu_shutdown();
  if ( !(devian_menu_main_menu_init()) )
    {
      fprintf(stderr, "dEvian: Bad, main menu could be regenerated ... going to crash ...\n");
    }
  //... Fix it
}

/* End of main menu section */



/* Menu of devians section */

static void _devian_menu_cb_source_infos(void *data, E_Menu *m, E_Menu_Item *mi)
{
  dEvian *devian;
  devian = data;

  devian_source_dialog_infos(devian);
}

static void _devian_menu_cb_source_viewer(void *data, E_Menu *m, E_Menu_Item *mi)
{
  dEvian *devian;
  devian = data;

  devian_source_viewer(devian);
}

static void _devian_menu_cb_devian_delete(void *data, E_Menu *m, E_Menu_Item *mi)
{
  dEvian *devian;

  devian = data;

  /* CHECK: if the devian was destroyed but the menu stays (key space) */
  if (!devian)
    return;
  if ((!devian->source) || (!devian->conf) || (!devian->menu))
    return;

  devian_devian_del(devian, 0);
}

static void _devian_menu_cb_set_container_box(void *data, E_Menu *m, E_Menu_Item *mi)
{
  
}

static void _devian_menu_cb_set_source_picture(void *data, E_Menu *m, E_Menu_Item *mi)
{

}

static void _devian_menu_cb_configure(void *data, E_Menu *m, E_Menu_Item *mi)
{
  dEvian *devian;

  devian = data;
  devian->dialog_conf = devian_config_dialog_devian(dEvianM->container, devian);
  e_object_ref(E_OBJECT(devian->dialog_conf));
}

static void _devian_menu_cb_edit_mode(void *data, E_Menu *m, E_Menu_Item *mi)
{
  e_gadman_mode_set(dEvianM->container->gadman, E_GADMAN_MODE_EDIT);
}

E_Menu *devian_menu_init(dEvian *devian)
{
  E_Menu *mn;
  E_Menu_Item *mi;

  DMAIN(("dEvian : menu"));

  mn = e_menu_new();

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Source Infos"));
  e_menu_item_callback_set(mi, _devian_menu_cb_source_infos, devian);
  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Source Viewer"));
  e_menu_item_callback_set(mi, _devian_menu_cb_source_viewer, devian);

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("delete dEvian"));
  e_menu_item_callback_set(mi, _devian_menu_cb_devian_delete, devian);

  mi = e_menu_item_new(mn);
  e_menu_item_separator_set(mi, 1);
  

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Container Box"));
  e_menu_item_radio_set(mi, 1);
  e_menu_item_radio_group_set(mi, 1);
  if ( devian->conf->container_type == CONTAINER_BOX )
      e_menu_item_toggle_set(mi, 1);
  e_menu_item_callback_set(mi, _devian_menu_cb_set_container_box, devian);
  /*
  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Bar"));
  e_menu_item_radio_set(mi, 1);
  e_menu_item_radio_group_set(mi, 1);
  if ( devian->conf->container_type == CONTAINER_BAR )
      e_menu_item_toggle_set(mi, 1);
  e_menu_item_callback_set(mi, _devian_menu_cb_set_container_bar, devian);
  */

  mi = e_menu_item_new(mn);
  e_menu_item_separator_set(mi, 1);

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Source Picture"));
  e_menu_item_radio_set(mi, 1);
  e_menu_item_radio_group_set(mi, 2);
  if ( devian->conf->source_type == SOURCE_PICTURE )
      e_menu_item_toggle_set(mi, 1);
  e_menu_item_callback_set(mi, _devian_menu_cb_set_source_picture, devian);
  /*
  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Text"));
  e_menu_item_radio_set(mi, 1);
  e_menu_item_radio_group_set(mi, 2);
  if ( devian->conf->container_type == SOURCE_TEXT )
      e_menu_item_toggle_set(mi, 1);
  e_menu_item_callback_set(mi, _devian_menu_cb_set_source_text, devian);
  */

  mi = e_menu_item_new(mn);
  e_menu_item_separator_set(mi, 1);

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Configure"));
  e_menu_item_callback_set(mi, _devian_menu_cb_configure, devian);

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("General"));
  e_menu_item_submenu_set(mi, dEvianM->main_menu);

  mi = e_menu_item_new(mn);
  e_menu_item_label_set(mi, _("Edit Mode"));
  e_menu_item_callback_set(mi, _devian_menu_cb_edit_mode, devian);

  DMAIN(("dEvian : menu fin"));

  return mn;
}

void devian_menu_shutdown(dEvian *devian)
{
  if (devian->menu)
    {
      e_menu_deactivate(devian->menu);
      e_object_del(E_OBJECT(devian->menu));
      devian->menu = NULL;
    }
}

/* End of devians's menu section */
