#include "dEvian.h"

static void _devian_source_del(dEvian *devian);

/* Public functions */

int devian_source_change(dEvian *devian, int source)
{
  /* CHECK: if already one source, remove it, except if its the one we want */
  if (devian->source)
    {
      if (devian->conf->source_type == source)
	return 0;
      else
	_devian_source_del(devian);
    }

  /* Add the new container to the devian */
  switch (source)
    {
    case SOURCE_PICTURE:
      {
	if ( devian_source_picture_add(devian) )
	  return 1;
	break;
      }

    case SOURCE_TEXT:
      {
	return 1;
      }

    case SOURCE_NO:
      {
	/* I want no source attached, thank you */
	return 0;
      }

    default : return 1;
    }
  
  return 0;
}

void devian_source_detach(dEvian *devian, int part)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      devian_data_picture_cache_detach(devian->source, part);
      break;
    case SOURCE_TEXT:
      break;
    }
  return;
}

int devian_source_refresh(dEvian *devian)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_change(devian->source);
    case SOURCE_TEXT:
      return 0;
    }

  return 0;
}

int devian_source_set_bg(dEvian *devian)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_set_bg(devian->source);
    case SOURCE_TEXT:
      return 0;
    }

  return 0;
}

int devian_source_viewer(dEvian *devian)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_viewer(devian->source);
    case SOURCE_TEXT:
      return 0;
    }

  return 0;
}

/*
  Get the evas objects of a source
  Return:
    - 1 on success
    - 0 on fail
*/
int devian_source_evas_object_get(dEvian *devian,
				  Evas_Object **obj0,
				  Evas_Object **obj1)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_evas_object_get(devian, obj0, obj1);
    case SOURCE_TEXT:
	break;
    case SOURCE_NO:
	break;
    }

  return 0;
}

int devian_source_original_size_get(dEvian *devian,
				   int part, int *w, int *h)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_original_size_get(devian->source,
						    part, w, h);
    case SOURCE_TEXT:
	break;
    case SOURCE_NO:
	break;
    }

  return 0;
}

char *devian_source_name_get(dEvian *devian, int part)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      return devian_source_picture_name_get(devian->source, part);
    case SOURCE_TEXT:
      break;
    }
  return NULL;
}

void devian_source_dialog_infos(dEvian *devian)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      devian_source_picture_dialog_infos(devian->source);
      break;
    case SOURCE_TEXT:
      break;
    }
  
  return;
}

void devian_source_timer_change_set(dEvian *devian, int active, int time)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_PICTURE:
      devian_source_picture_timer_change_set(devian->source, active, time);
      break;
    case SOURCE_TEXT:
      break;
    }
  
  return;
}

/* Private functions */

static void _devian_source_del(dEvian *devian)
{
  switch (devian->conf->source_type)
    {
    case SOURCE_NO: return;
    case SOURCE_PICTURE: devian_source_picture_del(devian->source);
    case SOURCE_TEXT: return;

    default: return;
    }
}
