#include "dEvian.h"

static int _cb_timer_picture_change(void *data);
static void _was_set_bg_add(Source_Picture *source, char *name);
static void _was_set_bg_purge(Source_Picture *source, int in_use);


/*
  Add a source to a devian
  Return:
   - 0 on success
   - 1 if fails
*/
int devian_source_picture_add(dEvian *devian)
{
  Source_Picture *source;

  DSOURCE(("dEvian: Source picture creation START\n"));

  source = E_NEW(Source_Picture, 1);
  source->devian = devian;
  
  /* Set new name for devian */
  devian_devian_set_name(devian, SOURCE_PICTURE, NULL);

  /* Attach picture */
  if ( !devian_data_picture_cache_attach(source, 0) )
    {
      devian_source_picture_del(source);
      return 1;
    }

  /* Timer to change picture */
  if (devian->conf->picture_timer_active)
    devian_source_picture_timer_change_set(source, 1, 0);

  source->was_set_bg = NULL;

  devian->conf->source_type = SOURCE_PICTURE;
  devian->source = source;

  DSOURCE(("dEvian: Source picture creation OK\n"));

  return 0;
}

void devian_source_picture_del(Source_Picture *source)
{
  devian_data_picture_cache_detach(source, 0);
  devian_data_picture_cache_detach(source, 1);

  if (source->timer)
    {
      ecore_timer_del(source->timer);
    }

  if (dEvianM->conf->sources_picture_set_bg_purge)
    _was_set_bg_purge(source, 0);

  source->devian->source = NULL;
  source->devian->conf->source_type = SOURCE_NO;

  E_FREE(source);

  DSOURCE(("dEvian: Source deleted\n"));

}

/*
  Change the picture of a picture source
  Return:
    - 1 on success
    - 0 on fail
*/
int devian_source_picture_change(Source_Picture *source)
{
  int old_part;

  old_part = devian_container_edje_part_get(source->devian);

  if (!devian_data_picture_cache_attach(source,
					!old_part))
    return 0;

  if(!devian_container_edje_part_change(source->devian))
      return 0;

  devian_container_resize_auto(source->devian);
			      
  return 1;
}

int devian_source_picture_set_bg(Source_Picture *source)
{
  E_Zone *zone;
  char *file = NULL;
  char *name;
  char buf[4096];

  zone = e_zone_current_get(dEvianM->container);
  if (!zone)
    return 0;

  if (!devian_container_edje_part_get(source->devian))
    {
      if (source->picture0)
	{
	  file = source->picture0->path;
	  name = source->picture0->picture_description->name;
	}
    }
  else
    {
      if (source->picture1)
	{
	  file = source->picture1->path;
	  name = source->picture1->picture_description->name;
	}
      else
	return 0;
    }
  if (!file)
    return 0;

  if (dEvianM->conf->sources_picture_set_bg_purge)
    _was_set_bg_purge(source, 1);

  if (!ecore_file_exists(file))
    {
      snprintf(buf, sizeof(buf),
               _(
                 "<hilight>File %s doesnt exists.</hilight><br><br>"
		 "This file is in dEvian's picture list, but it seems you removed it from the disk<br>"
		 "It cant be set as background, sorry."
                 ),
               file
               );
      e_module_dialog_show(_("dEvian Module Error"), buf);
      return 0;
    }

  if (!strstr(file, ".edj"))
    {
      if (ecore_file_app_installed("e17setroot"))
	{
	  Ecore_Exe *exe;
	  
	  snprintf(buf, 4096, "e17setroot -s %s", file);
	  DSOURCE(("Set background with %s", buf));
	  exe = ecore_exe_run(buf, NULL);
	  if (exe)
	    {
	      ecore_exe_free(exe);
	      if (dEvianM->conf->sources_picture_set_bg_purge)
		_was_set_bg_add(source, name);
	    }
	}
      else
	{
	  snprintf(buf, sizeof(buf),
		   _(
		     "<hilight>e17setroot needed</hilight><br><br>"
		     "%s is not an edje file !<br>"
		     "dEvian need e17setroot util from e_utils package to set you're picture as background"
		     "Please install it and try again"
		     ),
		   file
		   );
	  e_module_dialog_show(_("dEvian Module Error"), buf);
	  return 0;
	}
    }
  else
    { 
      DSOURCE(("Set edje background %s", file));

      if ((zone->container->num == 0) && (zone->num == 0) &&
	  (zone->desk_x_current == 0) && (zone->desk_y_current == 0))
	{
	  e_lib_background_set(strdup(file));
	}
      else
	{
	  e_lib_desktop_background_del(zone->container->num, zone->num, zone->desk_x_current, zone->desk_y_current);
	  e_lib_desktop_background_add(zone->container->num, zone->num, zone->desk_x_current, zone->desk_y_current, strdup(file));
	}
    }

  return 1;
}

int devian_source_picture_viewer(Source_Picture *source)
{
  char *file;
  char buf[4096];

  if (!devian_container_edje_part_get(source->devian))
    {
      if (source->picture0)
	{
	  file = source->picture0->path;
	}
      else
	return 0;
    }
  else
    {
      if (source->picture1)
	{
	  file = source->picture1->path;
	}
      else
	return 0;
    }
  if (!file)
    return 0;
  if (!ecore_file_exists(file))
    {
      snprintf(buf, sizeof(buf),
               _(
                 "<hilight>File %s doesnt exists.</hilight><br><br>"
		 "This file is in dEvian's picture list, but it seems you removed it from the disk<br>"
		 "It cant be set opened in a viewer, sorry."
                 ),
               file
               );
      e_module_dialog_show(_("dEvian Module Error"), buf);
      return 0;
    }  
  
  if (ecore_file_app_installed("exhibit"))
    {
      Ecore_Exe *exe;
      
      snprintf(buf, 4096, "exhibit %s", file);
      DSOURCE(("Viewer: %s", buf));
      exe = ecore_exe_run(buf, NULL);
      if (exe)
	ecore_exe_free(exe);
    }
  else
    {
      snprintf(buf, sizeof(buf),
	       _(
		 "<hilight>exhibit not found !</hilight><br><br>"
		 "Exhibit in an image viewer writen with Enlightenment Librairies. You can get it on E cvs"
		 )
	       );
      e_module_dialog_show(_("dEvian Module Error"), buf);
      return 0;
    }

  return 1;
}

int devian_source_picture_evas_object_get(dEvian *devian,
					  Evas_Object **picture0,
					  Evas_Object **picture1)
{
  Source_Picture *source;

  source = (Source_Picture *)devian->source;

  if ((!source->picture0) && (!source->picture1))
    return 0;

  if (source->picture0)
    *picture0 = source->picture0->picture;
  else
    *picture0 = NULL;
  if (source->picture1)
    *picture1 = source->picture1->picture;
  else
    *picture1 = NULL;

  return 1;
}

int devian_source_picture_original_size_get(Source_Picture *source,
					    int part, int *w, int *h)
{
  if (!source)
    return 0;

  if (part == -1)
    {
      if ( (part = devian_container_edje_part_get(source->devian)) == -1 )
	return 0;
    }

  if (!part && source->picture0)
    {
      *w = source->picture0->original_w;
      *h = source->picture0->original_h;
      return 1;
    }

  if (part && source->picture1)
    {
      *w = source->picture1->original_w;
      *h = source->picture1->original_h;
      return 1;
    }

  return 0;
}

void devian_source_picture_dialog_infos(Source_Picture *source)
{
  E_Dialog *dia;
  char buf[4096];
  char *path;
  int edje_part;

  edje_part = devian_container_edje_part_get(source->devian);
  if(!edje_part)
    path = source->picture0->path;
  else
    path = source->picture1->path;

  dia = e_dialog_new(dEvianM->container);
  if (!dia)
    return;

  snprintf( buf, 4096,
	    "<hilight>Informations</hilight><br>"
	    "Picture name : %s<br>"
	    "Picture path : %s<br>"
	    "<br>"
	    "Total pictures in local list : %d",
	    devian_source_picture_name_get(source, edje_part),
	    path,
	    evas_list_count(dEvianM->picture_list_local->pictures) );
  e_dialog_title_set(dia, "dEvian Module - Picture Informations");
  e_dialog_icon_set(dia, "enlightenment/e", 64);
  e_dialog_text_set(dia, buf);
  e_dialog_button_add(dia, _("Ok"), NULL, NULL, NULL);
  e_win_centered_set(dia->win, 1);
  e_dialog_show(dia);

}

char *devian_source_picture_name_get(Source_Picture *source, int part)
{
  char *name;
  name = NULL;

  if (part == -1)
    {
      part = devian_container_edje_part_get(source->devian);
    }

  if (!part && source->picture0)
    name = source->picture0->picture_description->name;
  
  if (part && source->picture1)
    name = source->picture1->picture_description->name;
  
  return name;
}

void devian_source_picture_timer_change_set(Source_Picture *source, int active, int time)
{
  if (time && (time < SOURCE_PICTURE_TIMER_MIN))
    return;
 
  source->devian->conf->picture_timer_active = active;
  if (!time)
    time = source->devian->conf->picture_timer_s;
  else
    source->devian->conf->picture_timer_s = time;

  if (!active)
    {
      if (source->timer)
	{
	  devian_container_edje_part_change_set(source->devian, 0);
	  ecore_timer_del(source->timer);
	  source->timer = NULL;
	}
      return;
    }

  if (source->timer)
    ecore_timer_del(source->timer);
  source->timer = ecore_timer_add(time,
				  _cb_timer_picture_change,
				  source);
  devian_container_edje_part_change_set(source->devian, 1);
}

/* Private functions */

static int _cb_timer_picture_change(void *data)
{
  Source_Picture *source;
  source = data;

  devian_source_picture_change(source);

  return 1;
}

static void _was_set_bg_add(Source_Picture *source, char *name)
{
  char *home;
  char *file_edj;

  file_edj = E_NEW(char, DEVIAN_MAX_PATH);
  home = e_user_homedir_get();
  snprintf(file_edj, DEVIAN_MAX_PATH, "%s/.e/e/backgrounds/%s.edj", home, name);
  free(home);

  source->was_set_bg = evas_list_append(source->was_set_bg, file_edj);
}

static void _was_set_bg_purge(Source_Picture *source, int in_use)
{
  char *file;
  int n;

  if (!evas_list_count(source->was_set_bg))
    return;

  while ( (n = evas_list_count(source->was_set_bg)) )
    {
      file = evas_list_data(source->was_set_bg);
      DSOURCE(("was_set_bg check %d %s", n, file));
      if (!strcmp(e_config->desktop_default_background, file))
	{
	  if (in_use)
	    {
	      if (n==1)
		break;
	      else
		{
		  source->was_set_bg = evas_list_remove(source->was_set_bg, file);
		  E_FREE(file);
		  continue;
		}
	    }
	  else
	    {
	      source->was_set_bg = evas_list_remove(source->was_set_bg, file);
	      E_FREE(file);
	      continue;
	    }
	}
      DSOURCE(("was_set_bg remove %s", file));
      if (ecore_file_exists(file))
	ecore_file_unlink(file);
      source->was_set_bg = evas_list_remove(source->was_set_bg, file);
      E_FREE(file);
    }

  return;
}
